/*
 * It receives data through SFPDP.
 * test: 16M recv.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/poll.h>
#include <time.h>
#include <sys/time.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"

#define MXDATA_SIZE	(32*1024)	// 32*1024 = 32K BLK
// #define MXDATA_SIZE	(16*1024*1024)	// 16*1024*1024 = 16M BLK

#define NUM_OF_CH 4
#define LOG_NUM 1050

static unsigned int recv_tot[NUM_OF_CH] = {0,0,0,0};	// total size of receive. 
static unsigned int recv_bufstrt[NUM_OF_CH];
static unsigned int recv_bufend[NUM_OF_CH];

extern int fd[4]; 

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();

extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);


unsigned int GetMSec()
{
	unsigned int uiCurTime;
	struct timeval tv;

	gettimeofday(&tv, NULL);
	uiCurTime = tv.tv_sec * 1000 + tv.tv_usec/1000;

	return uiCurTime;
}



void *monitor_thread(void* arg)
{
	int i; 

	while(1) {
		for( i=0; i<NUM_OF_CH; i++) {

			if( recv_bufend[i] == (MXDATA_SIZE-1) ) {
				printf("MNT: Recv Data size in Ch[%d]: %d\n", i, recv_tot[i] );
				printf("MNT: First Value of recv Data: 0x%08x\n", recv_bufstrt[i] );
				printf("MNT: End Value of recv Data:   0x%08x\n", recv_bufend[i] );
				recv_bufend[i] = 0; 
			}
		}
		usleep( 100 );
	}
}



void *recv_thread(void* arg)
{
	int ch = 0;
//	int nIndex = 0;
	int nRxSize;
	char pRxBuf[RX_FRMSIZE + 128];

	ch = *(int *)arg;

	recv_tot[ch]=0; 

	/* Data was received recursively, but no data is pending the read() */ 
	while(1) {
		nRxSize = read( fd[ch], pRxBuf, RX_FRMSIZE );

		recv_tot[ch] += nRxSize;

//		if( recv_tot[ch] < TX_MAXSIZE ) continue; 

		if( recv_tot[ch] < (4096*10) ) continue; 
		else {
			/* gpio check */
			printf("Tot Recv size in Ch[%d]: %d\n", ch, recv_tot[ch] );
			scGpioSetHigh7(); 
			scGpioSetLow7(); 

			recv_tot[ch]=0; 
		}
	} /*while*/

	return NULL;
}


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};

	pthread_t p_recv0;
	pthread_t p_recv1;
	pthread_t p_recv2;
	pthread_t p_recv3;
	pthread_t p_monitor;
	pthread_t p_gpio;

	int iRecvThreadID0;
	int iRecvThreadID1;
	int iSendThreadID0;
	int iRecvThreadID2;
	int iRecvThreadID3;
	int iMonitorThreadID;
//	int iGpioThreadID;

	system("rm -rf log*");

	scGpioInit();
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
	GEF_STATUS gpioStatus;

	if( sfpdp_init() == 0 ) printf("sfpdp_init..\n");
	else {
		printf( "sfpdp_init fails..\n");
		scGpioClose();
		return -1; 
	}

	sfpdp_start( 0, RXMODE_WITHOUT_PATTERN );
//	sfpdp_start( 1, RXMODE_WITHOUT_PATTERN );
//	sfpdp_start( 1, RXMODE_AFTER_PATTERN );
//	sfpdp_start( 2, RXMODE_AFTER_PATTERN );
//	sfpdp_start( 3, RXMODE_AFTER_PATTERN );
	printf("sfpdp_start..\n");

//	iMonitorThreadID = pthread_create( &p_monitor, NULL, monitor_thread, NULL );
	iRecvThreadID0 = pthread_create( &p_recv0, NULL, recv_thread, (void *)&chNum[0]);
/*
	iRecvThreadID1 = pthread_create( &p_recv1, NULL, recv_thread, (void *)&chNum[1] );
	iRecvThreadID2 = pthread_create( &p_recv2, NULL, recv_thread, (void *)&chNum[2]);
	iRecvThreadID3 = pthread_create( &p_recv3, NULL, recv_thread, (void *)&chNum[3]);
*/
	pthread_join(p_recv0, NULL);
/*
	pthread_join(p_recv1, NULL);
	pthread_join(p_recv2, NULL);
	pthread_join(p_recv3, NULL);
*/
//	pthread_join(p_monitor, NULL);

	scGpioClose();
	sfpdp_close();

	return 0;
}

