/*
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/poll.h>
#include <time.h>
#include <sys/time.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <semaphore.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"

#define RXBUFF_SIZE	(ONEK_BLK*40)
#define RXBUFF_HALF	(ONEK_BLK*20)

//#define SINGLE_DURATION_PKT_SIZE (40960*1)
//#define SINGLE_DURATION_PKT_SIZE (1920000)
//#define SINGLE_DURATION_PKT_SIZE (60800)
//#define GPIO_PKT_SIZE SINGLE_DURATION_PKT_SIZE

#define NUM_OF_CH 4
#define LOG_NUM 1050


static unsigned int frame_size[4] = {0,0,0,0};
static unsigned long int g_frame_size[4] = {0,0,0,0};

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();

extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);

extern GEF_GPIO_HDL gpioHdl;

sem_t g_sendSem[NUM_OF_CH];


unsigned int GetMSec()
{
	unsigned int uiCurTime;
	struct timeval tv;

	gettimeofday(&tv, NULL);

	uiCurTime = tv.tv_sec * 1000 + tv.tv_usec/1000;

	return uiCurTime;
}


void *timer_thread(void *arg)
{
	unsigned int uiStartTime = 0;
	unsigned int uiCurrTime = 0;
	int iTickInterVal = 1000;
	int uiTickCount = 0;
	int ch = 0;

	uiStartTime = GetMSec();
	while(1)
	{
		uiCurrTime = GetMSec();
		if( (uiCurrTime - uiStartTime) >= iTickInterVal )
		{
			uiTickCount++;
	//		printf("[%d] Rx[%d] %5.3f Gbps (%ld)\n", uiTickCount, ch, g_frame_size[ch] * 8   / 1000000000.f, frame_size[ch]);
	//		printf("[%d] Rx[0] %5.3f Gbps (%ld), Rx[1] %5.3f Gbps (%ld), Rx[2] %5.3f Gbps (%ld), Rx[3] %5.3f Gbps (%ld)\n", uiTickCount, g_frame_size[0] * 8   / 1000000000.f, frame_size[0], g_frame_size[1] * 8   / 1000000000.f, frame_size[1], g_frame_size[2] * 8   / 1000000000.f, frame_size[2], g_frame_size[3] * 8   / 1000000000.f, frame_size[3]);
			g_frame_size[0] = 0;
			g_frame_size[1] = 0;
			g_frame_size[2] = 0;
			g_frame_size[3] = 0;
			uiStartTime = uiCurrTime;
		}
		usleep(100000);
	}
	return NULL;
}


void *recv_thread(void* arg)
{
	int ch = 0;
	unsigned int buff[ONEK_BLK*10];
	unsigned int ret_len;
	int i = 0;
	int j = 0;
	unsigned long int checksum = 0;

	ch = *(int *)arg;

	/* If data is received recursively, the method is similar to the polling. */
	while(1) {
		if( ((j+1)%13) == 0 )
			ret_len = sfpdp_recv(ch, (char *)buff, RXBUFF_HALF, 0);
		else
			ret_len = sfpdp_recv(ch, (char *)buff, RXBUFF_SIZE, 0);
		printf("ret_len:%d.\n", ret_len);
/*
		printf("buff[0] =0x%x, buff[1] =0x%x \n", buff[0], buff[1] );
		printf("buff[2] = 0x%x \n", buff[2]);
*/
		if( buff[0] != 0x5555aaaa || buff[1] != 0x5555aaaa ) putchar('H');

		if( buff[2] != j ) {	// to validate the same buff block?
			putchar('J');
			j = buff[2];
		}

		if( (ret_len == RXBUFF_SIZE) || (ret_len == RXBUFF_HALF) ) {
			if(ch == 3) {
				scGpioSetHigh7();
				scGpioSetLow7();
			}

			i = 3;
			checksum = 0;
			// if(ret_len == 20484) ret_len = 20480;

			if( ret_len == 40960 ) {
				while( i<(ret_len/4)-1 ) {
					checksum += buff[i];
					i++;
				}
				if(checksum != buff[(ret_len/4)-1]) {
					putchar('C');
					printf("checksum = %d\n", checksum);
				}
			}
			//printf("checksum = %ld \n", checksum);

		}
		else	putchar('L');
/*
		if((checksum != 10237) && (checksum != 5117)) {
			putchar('C');
			printf("checksum = %d\n", checksum);
		}
*/
		j++;
		usleep(2000*1000);
	} /*while*/

	return NULL;
}


/*
void *gpio_thread(void* arg)
{
	int i = 0;
	int pin = 2;
	GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
	GEF_STATUS gpioStatus;
	struct timeval timeout;
	timeout.tv_sec = 300;
	timeout.tv_usec = 0;

	getchar();
//	gefGpioWaitForInterrupt(gpioHdl, &pinBitmap, &timeout);
	gefGpioWaitForInterrupt(gpioHdl, &pinBitmap, &timeout);
	gefGpioWaitForInterrupt(gpioHdl, &pinBitmap, &timeout);

	while(1) {
		gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout);
		for(i = 0; i < NUM_OF_CH; i++) {
			sem_post(&g_sendSem[i]);
		}
	}
	return NULL;
}
*/


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};
	int semIndex = 0;

	pthread_t p_recv0;
	pthread_t p_recv1;
	pthread_t p_recv2;
	pthread_t p_recv3;
	pthread_t p_tick_thread;
	pthread_t p_gpio;
	pthread_t p_send0;

	int iRecvThreadID0;
	int iRecvThreadID1;
	int iSendThreadID0;
	int iRecvThreadID2;
	int iRecvThreadID3;
	int iTickThreadID;
	int iGpioThreadID;

	system("rm -rf log*");

	scGpioInit();
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
	GEF_STATUS gpioStatus;

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ )
	{
		if(sem_init(&g_sendSem[semIndex], 0, 0) == -1) {
			printf("[%s] Semaphore Init ERROR\n", __FUNCTION__);
			return -1;
		}
	}

	if( sfpdp_init() == 0 ) printf("sfpdp_init..\n");
	else {
		printf( "sfpdp_init fails..\n");
		scGpioClose();
		return -1; 
	}

//	sfpdp_start( 0, RXMODE_WITHOUT_PATTERN );
	sfpdp_start( 1, RXMODE_WITHOUT_PATTERN );
//	sfpdp_start( 2, RXMODE_AFTER_PATTERN );
//	sfpdp_start( 3, RXMODE_AFTER_PATTERN );
	printf("sfpdp_start..\n");

/*	iGpioThreadID = pthread_create(&p_gpio, NULL, gpio_thread, NULL);
	iTickThreadID = pthread_create(&p_tick_thread, NULL, timer_thread, NULL);
	iRecvThreadID0 = pthread_create(&p_recv0, NULL, recv_thread, (void *)&chNum[0]);
	iRecvThreadID2 = pthread_create(&p_recv2, NULL, recv_thread, (void *)&chNum[2]);
	iRecvThreadID3 = pthread_create(&p_recv3, NULL, recv_thread, (void *)&chNum[3]);
*/
	iRecvThreadID1 = pthread_create(&p_recv1, NULL, recv_thread, (void *)&chNum[1]);
/*
	pthread_join(p_recv0, NULL);
	pthread_join(p_recv2, NULL);
	pthread_join(p_recv3, NULL);
*/
	pthread_join(p_recv1, NULL);

	scGpioClose();
	sfpdp_close();

	return 0;
}


//printf("recv(%d) : 0[0x%02x], 1[0x%02x], 2[0x%02x], 100[0x%02x], 500[0x%02x], 1500[0x%02x], 2047[0x%02x] \n", ret, buff[0], buff[1], buff[2], buff[100], buff[500], buff[1500], buff[2047]);
#if 0
if(buff[501] != 0x1f5)
{
	printf("addr = 0x%08x \n", &buff[0]);
	printf("recv(%d) : 0[0x%08x], 1[0x%08x], 2[0x%08x], 100[0x%08x], 250[0x%08x], 501[0x%08x], 502[0x%08x], 503[0x%08x], 504[0x%08x], 505[0x%08x], 506[0x%08x], 507[0x%08x], 510[0x%08x], 511[0x%08x] \n", ret, buff[0], buff[1], buff[2], buff[100], buff[250], buff[501], buff[502], buff[503], buff[504], buff[505], buff[506], buff[507], buff[510], buff[511]);
	return 0;
}
#endif
//printf("recv(%d) : 250[0x%08x], 251[0x%08x], 252[0x%08x], 253[0x%08x], 254[0x%08x], 255[0x%08x] \n", ret, buff[250], buff[251], buff[252], buff[253], buff[254], buff[255]);
//printf("recv(%d) : 0[0x%08x], 1[0x%08x], 2[0x%08x], 3[0x%08x] \n", ret, buff[0], buff[1], buff[2], buff[3]);
//printf("recv(%d) : 0[0x%08x], 1[0x%08x], 2[0x%08x], 100[0x%08x], 500[0x%08x], 1500[0x%08x], 2047[0x%08x] \n", ret, buff[0], buff[1], buff[2], buff[100], buff[500], buff[1500], buff[2047]);	


