/*
 * It receives data through SFPDP.
 * test: 16M recv.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/poll.h>
#include <time.h>
#include <sys/time.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"

#define RXBUF_LSIZE	(4*1024)	// 4*4 = 16K BLK
#define MXDATA_SIZE	(32*1024)	// 32*1024 = 32K BLK
// #define MXDATA_SIZE	(16*1024*1024)	// 16*1024*1024 = 16M BLK

//#define SINGLE_DURATION_PKT_SIZE (40960*1)
//#define SINGLE_DURATION_PKT_SIZE (1920000)
//#define SINGLE_DURATION_PKT_SIZE (60800)
//#define GPIO_PKT_SIZE SINGLE_DURATION_PKT_SIZE

#define NUM_OF_CH 4
#define LOG_NUM 1050

static unsigned int recv_tot[NUM_OF_CH] = {0,0,0,0};	// total size of receive. 
static unsigned int recv_bufstrt[NUM_OF_CH];
static unsigned int recv_bufend[NUM_OF_CH];

extern int fd[4]; 

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();

extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);


unsigned int GetMSec()
{
	unsigned int uiCurTime;
	struct timeval tv;

	gettimeofday(&tv, NULL);
	uiCurTime = tv.tv_sec * 1000 + tv.tv_usec/1000;

	return uiCurTime;
}



void *monitor_thread(void* arg)
{
	int i; 

	while(1) {
		for( i=0; i<NUM_OF_CH; i++) {

			if( recv_bufend[i] == (MXDATA_SIZE-1) ) {
				printf("MNT: Recv Data size in Ch[%d]: %d\n", i, recv_tot[i] );
				printf("MNT: First Value of recv Data: 0x%08x\n", recv_bufstrt[i] );
				printf("MNT: End Value of recv Data:   0x%08x\n", recv_bufend[i] );
				recv_bufend[i] = 0; 
			}
		}
		usleep( 100 );
	}
}



void *recv_thread(void* arg)
{
	int ch = 0;
	int nIndex = 0;
	int nRxSize;
	char pRxBuf[4096 + 128];

	ch = *(int *)arg;

	recv_tot[ch]=0; 

	/* If data is received recursively, the method is similar to the polling. */
	while(1) {
		nRxSize = sfpdp_recv_single( ch, pRxBuf, 4096 );
		printf(" RxSize = %d\n", nRxSize);
		if( nRxSize < 0 ) {
			/* read error */
			printf("Recv error in Ch[%d]: %d\n", ch, recv_tot[ch] );
			continue; 
		}
		else if( nRxSize == 0 ) { 
			continue; 
		}

//		printf("Receive date size: %d\n", nRxSize );
//		if( recv_tot[ch] == 0 ) {
//			recv_bufstrt[ch] = pRxBuf[0];	// start value of data block. 
//		}


		recv_tot[ch] += nRxSize;

		for( nIndex = 0 ; nIndex < nRxSize ; nIndex++ ) {
			if(pRxBuf[nIndex] != (char)(nIndex % 256)) {
				printf(" Err at Index:%2d, Tx:0x%02x, Rx:0x%02x\n",
					nIndex, (char)(nIndex % 256), pRxBuf[nIndex] );
			}
		}	
		
/*		if( recv_tot[ch] == MXDATA_SIZE ) {
			printf("Recv Data size in Ch[%d]: %d\n", ch, recv_tot[ch] );
			if( rtn > 1) { 
				recv_bufend[ch] = rbuf[(rtn/4)-1];		// end value. 
				printf("Recv end val.= 0x%08x, rtn: %d\n", recv_bufend[ch], rtn );
			}
			recv_tot[ch]=0; 
		}
*/
	} /*while*/

	return NULL;
}


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};

	pthread_t p_recv0;
	pthread_t p_recv1;
	pthread_t p_recv2;
	pthread_t p_recv3;
	pthread_t p_monitor;
	pthread_t p_gpio;

	int iRecvThreadID0;
	int iRecvThreadID1;
	int iSendThreadID0;
	int iRecvThreadID2;
	int iRecvThreadID3;
	int iMonitorThreadID;
//	int iGpioThreadID;

	system("rm -rf log*");

	scGpioInit();
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
	GEF_STATUS gpioStatus;

	if( sfpdp_init() == 0 ) printf("sfpdp_init..\n");
	else {
		printf( "sfpdp_init fails..\n");
		scGpioClose();
		return -1; 
	}

	sfpdp_start( 1, RXMODE_AFTER_PATTERN );	// ch, mode
	printf("sfpdp_start..\n");
/*
//	sfpdp_start( 0, RXMODE_WITHOUT_PATTERN );
//	sfpdp_start( 1, RXMODE_AFTER_PATTERN );
//	sfpdp_start( 2, RXMODE_AFTER_PATTERN );
//	sfpdp_start( 3, RXMODE_AFTER_PATTERN );
*/

//	iMonitorThreadID = pthread_create( &p_monitor, NULL, monitor_thread, NULL );
	iRecvThreadID1 = pthread_create(&p_recv1, NULL, recv_thread, (void *)&chNum[1] );
/*
	iRecvThreadID0 = pthread_create(&p_recv0, NULL, recv_thread, (void *)&chNum[0]);
	iRecvThreadID2 = pthread_create(&p_recv2, NULL, recv_thread, (void *)&chNum[2]);
	iRecvThreadID3 = pthread_create(&p_recv3, NULL, recv_thread, (void *)&chNum[3]);
*/
	pthread_join(p_recv1, NULL);
/*
	pthread_join(p_recv0, NULL);
	pthread_join(p_recv2, NULL);
	pthread_join(p_recv3, NULL);
	pthread_join(p_monitor, NULL);
*/

	scGpioClose();
	sfpdp_close();

	return 0;
}


//printf("recv(%d) : 0[0x%02x], 1[0x%02x], 2[0x%02x], 100[0x%02x], 500[0x%02x], 1500[0x%02x], 2047[0x%02x] \n", ret, buff[0], buff[1], buff[2], buff[100], buff[500], buff[1500], buff[2047]);
#if 0
if(buff[501] != 0x1f5)
{
	printf("addr = 0x%08x \n", &buff[0]);
	printf("recv(%d) : 0[0x%08x], 1[0x%08x], 2[0x%08x], 100[0x%08x], 250[0x%08x], 501[0x%08x], 502[0x%08x], 503[0x%08x], 504[0x%08x], 505[0x%08x], 506[0x%08x], 507[0x%08x], 510[0x%08x], 511[0x%08x] \n", ret, buff[0], buff[1], buff[2], buff[100], buff[250], buff[501], buff[502], buff[503], buff[504], buff[505], buff[506], buff[507], buff[510], buff[511]);
	return 0;
}
#endif
//printf("recv(%d) : 250[0x%08x], 251[0x%08x], 252[0x%08x], 253[0x%08x], 254[0x%08x], 255[0x%08x] \n", ret, buff[250], buff[251], buff[252], buff[253], buff[254], buff[255]);
//printf("recv(%d) : 0[0x%08x], 1[0x%08x], 2[0x%08x], 3[0x%08x] \n", ret, buff[0], buff[1], buff[2], buff[3]);
//printf("recv(%d) : 0[0x%08x], 1[0x%08x], 2[0x%08x], 100[0x%08x], 500[0x%08x], 1500[0x%08x], 2047[0x%08x] \n", ret, buff[0], buff[1], buff[2], buff[100], buff[500], buff[1500], buff[2047]);	


