/*
 * send message through sFPDP. 
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"


#define TXBUFF_SIZE	(TENK_BLK+(256*49))	// 10240 + 256*9 ?
//#define BUFF_SIZE	((256*4))
//#define BUFF_SIZE	30720

#define WRITE_COUNT	1
#define INF_LOOP	1
#define NUM_OF_CH	4

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);

extern GEF_GPIO_HDL gpioHdl;

unsigned int buff0[TXBUFF_SIZE];
unsigned int buff1[TXBUFF_SIZE];
unsigned int buff2[TXBUFF_SIZE];
unsigned int buff3[TXBUFF_SIZE];

sem_t g_sendSem[NUM_OF_CH];


void SleepUSec(long a_nUSec)
{
    struct timeval stTimeVal;

    stTimeVal.tv_sec = a_nUSec / 1000000;
    a_nUSec -= stTimeVal.tv_sec * 1000000;
    stTimeVal.tv_usec = a_nUSec;

    select(0, 0, 0, 0, &stTimeVal);
}


#if 0
void *send_thread(void* arg)
{
        int devfd;              /* device file id */
        char szDeviceFile[80];
        int logfd;              /* log file id */
        char logFile[80];
        int ch = 0;
        unsigned int buff[TXBUFF_SIZE];
        unsigned int buff2[TXBUFF_SIZE];
        unsigned int i, j = 0;
        unsigned int pattern = 0;
        int rtn = 0;
        int cnt = 0;
        int logstopflag = 0;

        int pin = 1;
        GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
        GEF_STATUS gpioStatus;
        struct timeval timeout;
        timeout.tv_sec = 300;
        timeout.tv_usec = 0;

        ch = *(int *)arg;
        if (ch == 0) {
                sprintf(szDeviceFile,"/dev/pcidrv");
                sprintf(logFile, "log0");
        }
        else {
                sprintf(szDeviceFile,"/dev/pcidrv%d" ch);
                sprintf(logFile, "log%d", ch);
        }

        memset(buff, 0, TXBUFF_SIZE);
        memset(buff2, 0, TXBUFF_SIZE);

        devfd = open (szDeviceFile, O_RDWR | O_SYNC);
        if( devfd <= 0 )
        {
                printf( "[%s] *** Device file open fail.\n", __FUNCTION__);
                sem_destroy(&g_sendSem[ch]);
                return NULL;
        }

	printf( "[%s] *** Device file open success...\n", __FUNCTION__);
        if( !(logfd = open(logFile, O_WRONLY | O_CREAT, 0644)) ) {
                printf("logFile Open Err\n");
        }

        for(i=0; i < BUFF_SIZE ; i++) {
                buff[i] = pattern;
                buff2[i] = pattern;
                pattern++;
        }

        buff[0] = 0x5555aaaa;
        buff[1] = 0x5555aaaa;

        while(1) {
                rtn = 0;
                gefGpioWaitForInterrupt(gpioHdl, &pinBitmap, &timeout);

                scGpioSetHigh6();
                scGpioSetLow6();

                buff[2] = j++;
                rtn += write( devfd, buff, BUFF_SIZE);
        }

        close(logfd);
        close(devfd);
        printf("***close()\n");

        sem_destroy(&g_sendSem[ch]);
        return NULL;
}
#else 


void *send_thread(void* arg)
{
	int fd0, fd1, fd2, fd3;
	char    szDeviceFile[80];
	int ch = 0;
        unsigned int i; 
	unsigned int j=0;	// index of transfer block 
	int rtn = 0;
	int cnt = 0;
	int logstopflag = 0;
	unsigned long int checksum[4] = {0, 0, 0, 0};

	int fp;
	char logFile[80];
	unsigned int * ptr0, ptr1, ptr2, ptr3;

	ch = *(int *)arg;

	memset( buff0, 0, TXBUFF_SIZE*4 );	// 0 is constatn byte. 
	memset( buff1, 0, TXBUFF_SIZE*4 );
	memset( buff2, 0, TXBUFF_SIZE*4 );
	memset( buff3, 0, TXBUFF_SIZE*4 );

	buff0[0] = 0x5555aaaa;
	buff0[1] = 0x5555aaaa;
	buff1[0] = 0x5555aaaa;
	buff1[1] = 0x5555aaaa;
	buff2[0] = 0x5555aaaa;
	buff2[1] = 0x5555aaaa;
	buff3[0] = 0x5555aaaa;
	buff3[1] = 0x5555aaaa;
	
	srand( time(NULL) );

	while(1) {
		sem_wait( &g_sendSem[ch] );

		checksum[0] = 0;
		checksum[1] = 0;
		checksum[2] = 0;
		checksum[3] = 0;

		for(i=3; i<(TENK_BLK-1); i++) {
			buff0[i] = rand() % 10;
			buff1[i] = rand() % 10;
			buff2[i] = rand() % 10;
			buff3[i] = rand() % 10;
			checksum[0] += buff0[i];
			checksum[1] += buff1[i];
			checksum[2] += buff2[i];
			checksum[3] += buff3[i];
		}

		buff0[i] = checksum[0];
		buff1[i] = checksum[1];
		buff2[i] = checksum[2];
		buff3[i] = checksum[3];

		rtn = 0;
		for( cnt=0; cnt<12; cnt++) {
			buff0[2]=j;
			buff1[2]=j;
			buff2[2]=j;
			buff3[2]=j;

			rtn += sfpdp_send( ch, (char *)buff0, TXBUFF_SIZE*4 );
			if(rtn == 0) {
				printf( "%d-th block, there is no data of sfpdp_send.\n", j );
				scGpioSetHigh7();
				scGpioSetLow7();
			}
			j++;
		}

		buff0[2]=j;
		buff1[2]=j;
		buff2[2]=j;
		buff3[2]=j;
/*
		rtn += sfpdp_send( 0, (char *)buff0, (20*1024));
		if(rtn == 0) {
			scGpioSetHigh7();
			scGpioSetLow7();
		}
		j++;
*/
		printf( "Amount of send data: %d. Index: %d.\n", rtn, j );

	} /*while*/

	return NULL;
}
#endif	/* send_thread */


void *gpio_thread(void* arg)
{
	int i = 0;
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap;
	GEF_STATUS gpioStatus;
	struct timeval timeout;

	pinBitmap = (0x0001 << pin);
//	timeout.tv_sec = 300;
	timeout.tv_sec = 3;	// timeout is 3 seconds
	timeout.tv_usec = 0;

//	getchar();
//	gefGpioWaitForInterrupt(gpioHdl, &pinBitmap, &timeout);

	while(1) {
		gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );

		printf( "gpio: interrupted or timeout(%d sec)..\n", timeout.tv_sec );
		sleep(1);
	
//		scGpioSetHigh7();
//		scGpioSetLow7();
//		SleepUSec(400*1);
		for( i=0; i < NUM_OF_CH; i++ )	sem_post( &g_sendSem[i] );
	}
	return NULL;
}


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};
	int semIndex = 0;

	pthread_t p_send0;
	pthread_t p_send1;
	pthread_t p_send2;
	pthread_t p_send3;
	pthread_t p_gpio;

	int iSendThreadID0;
	int iSendThreadID1;
	int iSendThreadID2;
	int iSendThreadID3;
	int iGpioThreadID;

	system("rm -rf log*");

	scGpioInit();

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ ) {
		if( sem_init( &g_sendSem[semIndex], 0, 0) == -1) {
			printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
			return -1;
		}
	}

	if( sfpdp_init() != 0 ) return -1;
	printf("sfpdp_init \n");

	iGpioThreadID = pthread_create( &p_gpio, NULL, gpio_thread, NULL);
	iSendThreadID0 = pthread_create( &p_send0, NULL, send_thread, (void *)&chNum[0] );
//	iSendThreadID1 = pthread_create( &p_send1, NULL, send_thread, (void *)&chNum[1] );
//	iSendThreadID2 = pthread_create( &p_send2, NULL, send_thread, (void *)&chNum[2] );
//	iSendThreadID3 = pthread_create( &p_send3, NULL, send_thread, (void *)&chNum[3] );

	pthread_join(p_gpio, NULL);
	pthread_join(p_send0, NULL);
//	pthread_join(p_send1, NULL);
//	pthread_join(p_send2, NULL);
//	pthread_join(p_send3, NULL);

//	scGpioClose();

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ )
	       	sem_destroy( &g_sendSem[semIndex] );

	sfpdp_close();

	return 0;
}

