/*
 * This is basic functions for sFPDP communication. 
 */
#include <stdio.h>
#include <string.h>             // strlen()
#include <unistd.h>             // read, write
#include <fcntl.h>              // open, close, O_RDWR, O_NONBLOCK
#include <sys/ioctl.h>
#include "sfpdp-api.h"

#define TX_BUFSIZE 40960
#define RX_BUFSIZE 4096


int fd[4]={0,0,0,0};	/* device file id */

int temp_size[4] = {0,0,0,0};
char temp[4][RX_BUFSIZE];
int recv_size[4] = {0,0,0,0};


int sfpdp_start(int ch, int mode)
{
	PCIDRV_INFO pi;

	pi.addr = 0x50000 + (0x10000*ch) + 0x18;
	if( mode == RXMODE_AFTER_PATTERN )	pi.val = 0x03600003; //After PATTERN, data transfered. 
	else if( mode == RXMODE_WITHOUT_PATTERN ) pi.val = 0x02600003; // all the data is transfered.
	else return -1;
 
	pi.val |= (0x1 << 2);		// for loopback.  
	ioctl( fd[ch], IOCTL_SET_REG, &pi);

/* 0x4C : Timeout Reg. Between packets the timeout is allowed. 
 * 0x8fc=2300d, Default value is 0x47e (=1150 = 5.75us).
 * if (packet delay > Timeout) at the end of Data, "DEAD_BEEF" is inserted. 
*/
	pi.addr = 0x50000 + (0x10000*ch) + 0x4c;
	pi.val = 0x8fc;
	ioctl( fd[ch], IOCTL_SET_REG, &pi);

//	pi.addr = 0x50000 + (0x10000*ch) + 0x0C;
//	pi.val = 0;		// Mormal operation.
//	ioctl(fd[3], IOCTL_SET_REG, &pi);

	return 0;
}


it sfpdp_send(int ch, char* buf, int size)
{
	int rtn;
	int idx = 0;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	while( size > 0 )
	{
		rtn = write( fd[ch], &buf[(TX_BUFSIZE)*idx], (size>TX_BUFSIZE ? TX_BUFSIZE : size) );
		if( rtn < 0) {
			printf( "[%s] write error in.\n", __FUNCTION__ ); 
			return -1;
		}
		size -= TX_BUFSIZE;
		idx++;
	}

        return( TX_BUFSIZE*idx );
}


int sfpdp_recv( int ch, char* buf, int size, int usrtimeout)
{
	int ret;
	int idx = 0;
	int idx2 = 0;
	int i = 0;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	if( temp_size[ch] >= size) {
		memcpy( buf, &temp[ch][(recv_size[ch]-temp_size[ch])], size);
		temp_size[ch] -= size;
		size = 0;
	}
	else if( temp_size[ch] > 0) {
		memcpy(buf, &temp[ch][(recv_size[ch]-temp_size[ch])], temp_size[ch]);
		size -= temp_size[ch];
		idx = temp_size[ch];
		temp_size[ch] = 0;
	}

	while( size > 0) {
		ret = read( fd[ch], temp[ch], RX_BUFSIZE);
//		ret = read( fd[ch], temp[ch], usrtimeout);
		if( !(0<ret) ) {
			printf("[%s] Read error.\n", __FUNCTION__ );
			return 0;
		}

		if( ret <= 40960-2048+4 ) ret -= 4;

		if( ret > 0) {
			recv_size[ch] = ret;
			if( size < recv_size[ch] ) {
				memcpy( &buf[(idx)+(idx2)], temp[ch], size );
				temp_size[ch] = recv_size[ch] - size;
				size -= size;	// size = 0;
			}
			else {
				memcpy( &buf[(idx)+(idx2)], temp[ch], recv_size[ch] );
				size -= recv_size[ch];
				temp_size[ch] = 0;
			}
			idx2 += ret;
		}
	} //while

	printf("[Amount of Recv(%d)]\n", idx2 );
	return idx2;
}


int sfpdp_init()
{
	int i = 0;

	if( 0 < (fd[0] = open("/dev/scsfpdp0", O_RDWR|O_SYNC)) ) printf("**open(fd0)\n");
	else {
		printf("***open() fail.\n");
		return -1; 
	}

	if(0 < (fd[1] = open("/dev/scsfpdp1", O_RDWR|O_SYNC)) ) printf("**open(fd1)\n");
	else {
		printf("***open() fail.\n");
		return -1; 
	}

	if(0 < (fd[2] = open("/dev/scsfpdp2", O_RDWR|O_SYNC)) ) printf("**open(fd2)\n");
	else {
		printf("***open() fail.\n");
		return -1; 
	}

	if(0 < (fd[3] = open("/dev/scsfpdp3", O_RDWR|O_SYNC)) ) printf("**open(fd3)\n");
	else {
		printf("***open() fail.\n");
		return -1; 
	}

	for(i = 0; i < 4; i++)	memset( temp[i], 0, RX_BUFSIZE);

        return 0;
}


int sfpdp_close()
{
	int i = 0;

	for(i = 0; i < 4; i++)	close(fd[i]);
	return 0;
}


int sfpdp_reset(int ch)
{
	PCIDRV_INFO pi;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	pi.addr = 0x50000 + (0x10000*ch) + 0x18;
	pi.val = 0x00600003;
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	usleep(1000*100);

	pi.addr = 0x50000 + (0x10000*ch) + 0x0c;	// Master Control Register
	pi.val = 0x01;					// Reset the sFPDP channel
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	usleep(1000*100);

	pi.addr = 0x50000 + (0x10000*ch) + 0x0c;
	pi.val = 0x00;					// Normal operations
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	usleep(1000*100);

	return 0;
}


int sfpdp_bd_reset()
{
//	PCIDRV_INFO pi;		// there is no pi setting? 

	if( !(0<fd[3]) ) {
		printf( "fd[%d] is not opened in [%s]..", 3, __FUNCTION__ );
		return -1;
	}

	ioctl( fd[3], IOCTL_RESET, NULL );
	return 0;
}

