/*
 * This is to send some data through 3 channels of an SFPDP Card
 * using Hanwha sinario..
 * At 19 July 2021, it was modified by Jeongki Kim.
 */

#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write, usleep()
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"

#define NUM_OF_CH	4


extern int scGpioPinInterptInit();
// extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6();
extern int scGpioSetLow6();

extern GEF_GPIO_HDL gpioHdl;

sem_t sendSem[NUM_OF_CH];
// pthread_mutex_t TXlock, GXlock;			// mutex TX lock 


void *gpio_thread( void* arg )		// arg : Intrpt GPIO input pin #: 0 is MCP?
{
	GEF_GPIO_PIN_BITMAP pinBitmap;
	int i;
	int gnum = *(int *)arg;

#ifdef _DEBUG_
#endif
	GEF_STATUS gret; 
	struct timeval timeout;

	timeout.tv_sec = 1;	// timeout (seconds)
	timeout.tv_usec = 0;

	pinBitmap = (0x0001 << gnum );

	while(1) {
		printf( "Press Enter key to continue..\n" );
		getchar();

		for(i=0; i<1; i++) {
			printf("GPIO Pinmap: 0x%08X.\n", pinBitmap );
			gret = gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );
			if( gret == GEF_STATUS_SUCCESS ) 
				printf("GPIO(pin#%d) Interrupted..\n", gnum );
			if( gret == GEF_STATUS_TIMED_OUT ) 
				printf("GPIO(pin#%d) Timed out..\n", gnum );
#ifdef _DEBUG_
#else
//			gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, NULL );
#endif
		}

		for( i=0; i<NUM_OF_CH; i++ ) sem_post( &sendSem[i] );
	}
	return NULL;
}


int send_start_packet( int ch, int size )
{
	unsigned int	match_pattern = 0x5555AAAA;
	unsigned char	tbuf[HAN_BUFSIZE+16];
	int i, rtn;

	memcpy( (void*)&tbuf[0], (void*)&match_pattern, sizeof(int) );
	memcpy( (void*)&tbuf[4], (void*)&match_pattern, sizeof(int) );

	for( i=8; i<size; i++)  tbuf[i] = i % 256;

	rtn = sfpdp_send( ch, tbuf, size );
	if( rtn == size ) return size;
	else return -1;
}



unsigned int send_control_packet( int ch, int size )
{
	unsigned int	pcktsize;		/* next data packet size */
	unsigned char	tbuf[HAN_BUFSIZE+16];
	int i, rtn;

//	pcktsize = ( rand()%16 ) * 64*1024 + 64*1024;
	pcktsize = 1 * 1024*100;			// Data Packet Size.
	
#ifdef _DEBUG_
	printf( "[%s]>Ch[%d], TX Data Packet size:%d Bytes.\n", __func__, ch, pcktsize );
#endif
	memcpy( (void*)&tbuf[0], (void*)&pcktsize, sizeof(int) );
	for( i=4; i<size; i++)  tbuf[i] = i % 256;

	rtn = sfpdp_send( ch, tbuf, size );
	if( rtn == size ) return pcktsize;
	else return 0;
}


unsigned int send_data_packet( int ch, int size )
{
	int i, rtn;
	unsigned char	tbuf[HAN_BUFSIZE+16];

	for( i=0; i < size; i++)  tbuf[i] = i % 256;

	rtn = sfpdp_send( ch, tbuf, size );
	if( rtn == size ) return size;
	else return -1;
}


int send_end_packet( int ch, int size )
{
	unsigned int	end_pattern = 0xE7E7E7E7;
	unsigned char	tbuf[HAN_BUFSIZE+16];
	int rtn, i;

	memcpy( (void*)&tbuf[0], (void*)&end_pattern, sizeof(int) );
	memcpy( (void*)&tbuf[4], (void*)&end_pattern, sizeof(int) );

	for( i=8; i<size; i++)  tbuf[i] = i % 256;

	rtn = sfpdp_send( ch, tbuf, size );
	if( rtn == size ) return size;
	else return -1;
}


/* It sends data packets through the SFPDP ch (="arg"). */

void *send_thread(void* arg)
{
	unsigned int npcktsize; 			// for next packet size
	int i, rtn;
//	unsigned char tbuf[HAN_BUFSIZE+64];		// more than 2 MB

	int ch = *(int *)arg;
	srand( time(NULL) );

	while(1) {
		sem_wait( &sendSem[ch] );

		scGpioSetHigh6();

		rtn = send_start_packet( ch, (BASIC_PKTSIZE+8) );
		if( rtn < 0) return NULL;

		npcktsize = send_control_packet( ch, (BASIC_PKTSIZE+64) );
		if( !(npcktsize > 0) ) return NULL;
//		usleep(500);

		for( i=0; i<10; i++ ) {
			rtn = send_data_packet( ch, npcktsize );
			if( rtn < 0) return NULL;
			usleep(500);
		}

		rtn = send_end_packet( ch, (BASIC_PKTSIZE+8) );
		if( rtn < 0) return NULL;

		scGpioSetLow6();

		printf( "Ch[%d], TX Data Packet size:%d Bytes.\n", ch, npcktsize );
#ifdef _DEBUG_
#endif
	}
	return NULL;
}


/*
int get_main_args( int argc, char **argv )
{
	int chno;

	if( argc != 2 ) printf( "Usage: %s [# of ch] \n\n", argv[0] );
	if( argc > 2 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the number of channels to send simultaniously (1 to %d): ", NUM_OF_CH );
			scanf( "%d", &chno );
		}
		else if( argc == 2 ) {
			chno = atoi( argv[1] );
		}

		if( (chno<1) || ( chno>NUM_OF_CH) ) {
			printf( "Range of Ch is out : %d\n", chno );
			argc =1;
			continue; 
		}
		else break;
	}
	return chno;
}
*/


int init_sema()
{
	int i;

	for( i=0; i<NUM_OF_CH; i++) {
		if( sem_init( &sendSem[i], 0, 0 ) == -1 ) {
			printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
			return -1;
		}
	}
}


int destroy_sema()
{
	int i;

	for( i=0; i<NUM_OF_CH; i++) sem_destroy( &sendSem[i] );
}



int main(int argc, char **argv)
{
	int gpin =1;		// GPIO interpt pin.
//	int numofch;
	int chnum[NUM_OF_CH] = {0, 1, 2, 3};

	pthread_t p_gpio;
	pthread_t p_send0;
	pthread_t p_send1;
	pthread_t p_send2;
	pthread_t p_send3;

	int igpioID;
	int isendID;

//	numofch= get_main_args( argc, argv );
	printf( "Execute: %s Num of Chs[%d] to sent packets thru SFPDP card..\n", argv[0], NUM_OF_CH-1 );

	/* First, GPIO initialize */
	scGpioPinInterptInit( gpin );	// using gpio #gnum as interrupt.
	init_sema();
//	pthread_mutex_init( &TXlock, NULL );
//	pthread_mutex_init( &GXlock, NULL );

	if( sfpdp_all_open() <0 ) return -1; 
	if( sfpdp_all_bdbuf_clear() <0 ) return -1; 

	igpioID = pthread_create( &p_gpio, NULL, gpio_thread, (void *)&gpin );
	isendID = pthread_create( &p_send0, NULL, send_thread, (void *)&chnum[0] );
	isendID = pthread_create( &p_send1, NULL, send_thread, (void *)&chnum[1] );
	isendID = pthread_create( &p_send2, NULL, send_thread, (void *)&chnum[2] );
//	isendID = pthread_create( &p_send3, NULL, send_thread, (void *)&chnum[3] );

	printf("SFPDP send start ..\n");

//	pthread_join(p_send3, NULL);
	pthread_join(p_send2, NULL);
	pthread_join(p_send1, NULL);
	pthread_join(p_send0, NULL);
	pthread_join(p_gpio, NULL);

	/* Finalize */
	scGpioClose();
	destroy_sema();

	sfpdp_all_close();

	return 0;
}


