/*
 * This test program recieves some data through an SFPDP card.
 * To confirm the integrity of the data, it compares two checksum values, 
 * one of them is a value received, and other is the value calculated from the data received.
 *
 * At 27 May 2021, modified by Jeongki Kim, SEEDCORE.
 */
#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"


extern int scGpioInit();
extern int scGpioClose();
extern int scGpioSetHigh6();
extern int scGpioSetLow6();
extern int scGpioSetHigh7();
extern int scGpioSetLow7();

extern GEF_GPIO_HDL gpioHdl;

static int RXsize=4096;


void *recv_thread(void* arg)
{
	int ch=0, i;
	int nIndex;
	int cntblk=0, retlen; 
	int totrecv=0;
//	unsigned char	dot=0x2E;
	unsigned char	rbuf[RX_BUFSIZE+16];

	unsigned int	calchksum=0, rxchksum=0;
#ifdef _DEBUG_
//	struct timeval mytime; 
#endif

	ch = *(int *)arg;

	while(1) {

		retlen = sfpdp_recv( ch, (char *)rbuf, RXsize, 0);
//		retlen = sfpdp_recv_single( ch, (char *)rbuf, 0);
		scGpioSetHigh7();
#ifdef _DEBUG_
//		gettimeofday( &mytime, NULL ); 
//		printf("\nCurr time(usec) in RX_user: %ld \n", mytime.tv_usec );
		if( retlen<0 ) {
			printf("\nERR: channel link RX err, retlen: %d\n", retlen );
			return NULL; 
		}
#endif
		totrecv += retlen; 

		/* to receive & calcurate check-sum of RX data */
#ifdef _DEBUG_
#endif
		memcpy( (void*)&rxchksum, (void*)&rbuf[retlen-sizeof(int)], sizeof(int) );
		scGpioSetLow7();

		calchksum = 0;
		for( i=0; i<(retlen-sizeof(int)); i++ ) calchksum += rbuf[i];
		
		if( calchksum != rxchksum ) 
			printf(">>ERR: Calculated Chksum=0x%X, RX Chksum=0x%X\n", calchksum, rxchksum); 

		printf("CH[%d] RX size: %d Bytes\n", ch, retlen );
		printf("Ch[%d] RX Total size: %d Bytes.\n\n", ch, totrecv );
		printf("Calculated chksum: 0x%X, RX chksum: 0x%X\n", calchksum, rxchksum ); 
//		putchar( dot );
//
	}

	return NULL;
}


int get_main_arg( int argc, char **argv )
{
	int chno;

	if( argc != 3 ) printf( "Usage: %s [ch#] [RXsize]\n\n", argv[0] );
	if( argc > 3 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the channel# and size to recieve data..\n" );
			printf( "Range of the Channel number is from 0 to %d: ", NUM_OF_DMA );
			scanf( "%d", &chno );
			printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
			scanf( "%d", &RXsize );
		}
		else if( argc == 2 ) {
			chno = atoi( argv[1] );
			printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
			scanf( "%d", &RXsize );
		}
		else if( argc == 3 ) {
			chno = atoi( argv[1] ); 
			RXsize = atoi( argv[2] ); 
		}

		if( (chno<0) || (chno>NUM_OF_DMA) || (RXsize<1) || (RXsize >TX_BUFSIZE*10) ) {
			printf( "Range of Input is over: Ch[%d], size:%d\n", chno, RXsize );
			argc = 1;
			continue; 
		}
		break; 
	}
	return chno;
}



int main(int argc, char **argv)
{
	int semIndex = 0;
	int chno;
	int irecvID;
	pthread_t p_recv;

	chno = get_main_arg( argc, argv );

	printf( "Execute: %s ch[%d] %d(RXsize)..\n", argv[0], chno, RXsize );

	/* First, GPIO initialize */
	scGpioInit( );

	if( sfpdp_ch_open( chno) != 0 )				return -1;

	/* 0x01000000 mode is "after pattern, recieve data */
//	if( sfpdp_init_control( chno, 0x01000000 ) != 0 )	return -1;

	sfpdp_bdbuf_clear( chno);
	printf("RX start ..\n");

	irecvID = pthread_create( &p_recv, NULL, recv_thread, (void *)&chno );

	pthread_join(p_recv, NULL);

	/* Finalize */
	scGpioClose( );
	sfpdp_ch_close( chno);

	return 0;
}

