/*
 * send & recv message through an SFPDP Card.
 */

#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write, usleep()
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"


extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6();
extern int scGpioSetLow6();

extern GEF_GPIO_HDL gpioHdl;

static int TXsize=4096;

sem_t sendSem[NUM_OF_DMA];


void *gpio_ch_thread(void* arg)
{
	GEF_GPIO_PIN_BITMAP pinBitmap;
	int i;
	static int gnum = 0;		// Intrpt GPIO input pin #0: MCP
	int ch = *(int *)arg;

#ifdef _DEBUG_
	GEF_STATUS	gret	= GEF_STATUS_SUCCESS; 
	struct timeval	timeout;

	timeout.tv_sec = 2;		// timeout is ? seconds
	timeout.tv_usec = 0;
#endif

	pinBitmap = (GEF_GPIO_PIN_BITMAP)(1 << gnum );

	while(1) {
		printf("Press ENTER key to get a GPIO Interpt..\n" );
		getchar();

		for(i=0; i<1; i++) {
#ifdef _DEBUG_
			printf("GPIO Pinmap: 0x%08X.\n", pinBitmap );
			gret = gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );
			if( GEF_GET_ERROR(gret) == GEF_STATUS_SUCCESS ) 
				printf("GPIO(pin#%d) Interrupted..\n", gnum );
			if( GEF_GET_ERROR(gret) == GEF_STATUS_TIMED_OUT ) 
				printf("GPIO(pin#%d) Timed out..\n", gnum );
#else
			gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, NULL );
#endif
		}

		sem_post( &sendSem[ch] );
//		printf("sendSem[%d] value: 0x%X\n", ch, sendSem[ch] );
	}
	return NULL;
}



/* It sends data with ? size through the SFPDP ch (="arg"). */
void *send_thread(void* arg)
{
	int ch=0;
	int i, rtn=0, cnt = 0;
	int totsend=0;		// total size of send
	unsigned int	chksum=0;		// for packet data integrity 
	unsigned char	tbuf[TX_BUFSIZE+64];	// 16 KB
	int count = 0;
//	struct timeval mytime; 

	ch = *(int *)arg;
	srand( time(NULL) );

	while(1) {
		chksum = 0;
		tbuf[0] = 0xAA;
		tbuf[1] = 0xAA;
		tbuf[2] = 0x55;
		tbuf[3] = 0x55;
		tbuf[4] = 0xAA;
		tbuf[5] = 0xAA;
		tbuf[6] = 0x55;
		tbuf[7] = 0x55;
		tbuf[8] =cnt;

		for(i=0; i < (TXsize-sizeof(int)); i++) {
//			tbuf[i] = rand() % 10;
			tbuf[i] = i % 256;
		}
		for(i=0; i < (TXsize-sizeof(int)); i++)  chksum += tbuf[i];

		memcpy( (void*)&tbuf[i], (void*)&chksum, sizeof(int) );

		sem_wait( &sendSem[ch] );
		scGpioSetHigh6();

		rtn = sfpdp_send( ch, tbuf, TXsize );
//		rtn = sfpdp_send_with_swdv( ch, (char *)tbuf, TXsize );
//		gettimeofday( &mytime, NULL );
//		printf( "\n Curr time (usec) TX_user: %ld \n", mytime.tv_usec );

#ifdef _DEBUG_
//		print_buff( tbuf, TXsize );
		if( !(0<rtn) ) {
			printf( "%d-th block, there is no data of sfpdp_send.\n", cnt );
		}
		printf( "CH[%d] real TXsize:%d Bytes (Req TXsize:%d)\n", ch, rtn, TXsize );
#endif
		totsend += rtn;

		scGpioSetLow6();
#ifdef _DEBUG_
		printf("Ch[%d] Total TX Size: %d Bytes\n", ch, totsend );
		printf("Ch[%d] TX Check-sum value: 0x%X\n", ch, chksum );
#endif
	} // while

	return NULL;
}


int get_main_args( int argc, char **argv )
{
	int chno;

	if( argc != 3 ) printf( "Usage: %s [ch#] [TXsize]\n\n", argv[0] );
	if( argc > 3 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the channel and size to transfer data..\n" );
			printf( "Range of the Channel number is from 0 to %d: ", NUM_OF_DMA );
			scanf( "%d", &chno );

			printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
			scanf( "%d", &TXsize );
		}
		else if( argc == 2 ) {
			chno = atoi( argv[1] );
			printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
			scanf( "%d", &TXsize );
		}
		else if( argc == 3 ) {
			chno = atoi( argv[1] ); 
			TXsize = atoi( argv[2] ); 
		}

		if( (chno<0) || (chno>NUM_OF_DMA) || (TXsize<1) || (TXsize >TX_BUFSIZE*10) ) {
			printf( "Range of the data is over: %d\n", TXsize );
			argc =1;
			continue; 
		}
		else break;
	}
	return chno;
}


int init_semas()
{
	int isem; 

	for( isem=0; isem < NUM_OF_DMA; isem++ ) {
		if( sem_init( &sendSem[isem], 0, 0) == -1) {
			printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
			return -1;
		}
	}
	return 0;
}


void destroy_semas()
{
	int isem; 

	for( isem=0; isem < NUM_OF_DMA; isem++ )
		sem_destroy( &sendSem[isem] );
}



int main(int argc, char **argv)
{
	int isem =0;
	int chno =0;

	pthread_t p_gpio;
	pthread_t p_send;

	int igpioID;
	int isendID;

	chno = get_main_args( argc, argv );

	printf( "Execute: %s Ch[%d] %d(TXsize) through SFPDP card..\n", argv[0], chno, TXsize );

	/* First, GPIO initialize */
	if( scGpioInit() )  return -1;	// using gpio #gnum as interrupt.
	if( init_semas() )  return -1;

	if( sfpdp_ch_open(chno) != 0 )			return -1;
//	if( sfpdp_init_control( chno, 0 ) != 0 )	return -1;
	sfpdp_bdbuf_clear( chno);

	igpioID = pthread_create( &p_gpio, NULL, gpio_ch_thread, (void *)&chno );
	isendID = pthread_create( &p_send, NULL, send_thread, (void *)&chno );

//	printf("SFPDP send start ..\n");

	pthread_join(p_send, NULL);
	pthread_join(p_gpio, NULL);

	/* Finalize */
	scGpioClose();
	destroy_semas();

	sfpdp_ch_close(chno);

	return 0;
}


