/*
 * This test program recieves some data thru 1 ch of an SFPDP card
 * using Hanwha Sinario. 
 *
 * At 19 July 2021, modified by Jeongki Kim, SEEDCORE.
 */

#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"

#define NUM_OF_CH	4

extern int scGpioInit();
extern int scGpioClose();
extern int scGpioSetHigh7();
extern int scGpioSetLow7();

extern GEF_GPIO_HDL gpioHdl;

// pthread_mutex_t RXlock;		// mutex lock for RX


int recv_start_packet( int ch, int size )
{
	int rtn;
	unsigned char	rbuf[HAN_BUFSIZE+16];

	rtn = sfpdp_recv( ch, rbuf, size, 0 );

	if( rtn != size ) return -1;
       
#ifdef _DEBUG_		
	printf("At Ch[%d] Start Packet[%d] is received..\n", ch, rtn );
//	print_buff( rbuf, size );
#endif
	return size;
}


unsigned int recv_control_packet( int ch, int size )
{
	unsigned int pcktsize;	  /* next data packet size */
	unsigned char	rbuf[HAN_BUFSIZE+16];
	int i, rtn;

	rtn = sfpdp_recv( ch, rbuf, size, 0 );

	if( rtn != size ) return -1;

	memcpy( (void*)&pcktsize, (void*)&rbuf[0], sizeof(int) );

#ifdef _DEBUG_
	printf("At Ch[%d] Control Packet[%d] is received..\n", ch, rtn );
	printf("\t Size of Next Data Packet: %d.\n", pcktsize );
//	print_buff( rbuf, size );
#endif

	return pcktsize;
}


unsigned int recv_data_packet( int ch, int size )
{
	int i, rtn;
	unsigned char	rbuf[HAN_BUFSIZE+16];

	rtn = sfpdp_recv( ch, rbuf, size, 0 );

	if( rtn != size ) return -1;
	
#ifdef _DEBUG_		
	printf("At Ch[%d], Data Packet[%d] is received..\n", ch, rtn );
	printf("\t Size of Data Packet: %d.\n", rtn );
//	print_bigbuff( rbuf, size );
#endif

	return size;
}


int recv_end_packet( int ch, int size )
{
	int rtn;
	unsigned char	rbuf[HAN_BUFSIZE+16];

	rtn = sfpdp_recv( ch, rbuf, size, 0 );

	if( rtn != size ) return -1;
       
#ifdef _DEBUG_		
	printf("At Ch[%d], End Packet[%d] is received..\n", ch, rtn );
//	print_buff( rbuf, size );
#endif
	return size;
}


void *recv_thread(void* arg)
{
	int i, rtn, npcktsize=0;
	int totrecv=0;
//	unsigned char	rbuf[HAN_BUFSIZE+16];

	int ch = *(int *)arg;

	while(1) {

		rtn = recv_start_packet( ch, (BASIC_PKTSIZE+8) );
		if( rtn < 0) return NULL;
		else totrecv += rtn;

		npcktsize = recv_control_packet( ch, (BASIC_PKTSIZE+64) );
		if( !(npcktsize > 0) ) return NULL;
		else totrecv += rtn;

		rtn = recv_data_packet( ch, npcktsize );
		if( rtn < 0) return NULL;
		else totrecv += rtn;

		rtn = recv_end_packet( ch, (BASIC_PKTSIZE+8) );
		if( rtn < 0) return NULL;
		else totrecv += rtn;

//		scGpioSetHigh7();
		printf( "\t RX Data packet size: %d Bytes.\n", npcktsize );
#ifdef _DEBUG_
		printf( "Ch[%d], Total RX Packet size:%d Bytes.\n\n", ch, totrecv );
#endif
//		scGpioSetLow7();
	}

	return NULL;
}


int get_main_arg( int argc, char **argv )
{
	int chno;

	if( argc != 2 ) printf( "Usage: %s [ch#] \n\n", argv[0] );
	if( argc > 2 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the channel# (from 0 to %d): ", (NUM_OF_DMA-1) );
			scanf( "%d", &chno );
		}
		else if( argc == 2 ) {
			chno = atoi( argv[1] );
		}

		if( (chno<0) || (chno>(NUM_OF_DMA-1)) ) {
			printf( "Range of Input is over: Ch[%d].\n", chno );
			argc = 1;
			continue; 
		}
		break; 
	}
	return chno;
}



int main(int argc, char **argv)
{
	int chnum[NUM_OF_CH] = {0, 1, 2, 3};
	int chno;
	int irecvID;
	pthread_t p_recv0;

	chno = get_main_arg( argc, argv );

	printf( "Execute: %s Num of chs[%d] to recv packets..\n", argv[0], NUM_OF_CH-1 );

	/* First, GPIO initialize */
	scGpioInit( );

	if( sfpdp_ch_open( chno ) != 0 )	return -1;
	if( sfpdp_bdbuf_clear( chno ) < 0 )	return -1;
//	pthread_mutex_init( &RXlock, NULL );

	/* 0x01000000 mode is "after pattern, recieve data */
//	if( sfpdp_init_control( chno, 0x01000000 ) != 0 )	return -1;

	printf("RX start ..\n");

	irecvID = pthread_create( &p_recv0, NULL, recv_thread, (void *)&chno );

	pthread_join( p_recv0, NULL);

	/* Finalize */
	scGpioClose( );
	sfpdp_ch_close( chno );

	return 0;
}

