/*
 * send data through sFPDP. 
 * test: 16M data send.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"


/*
#define TXBUF_LSIZE	(4*1024*1024)		// 4M x 4 = 16MB
#define TXBUF_SIZE	TENK_BLK
#define BUF_SIZE	((256*4))
#define BUF_SIZE	30720
#define TXNUM_BLK	8	// the number of Tx data
*/

#define WRITE_COUNT	1
#define INF_LOOP	1
#define NUM_OF_CH	4

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);

extern GEF_GPIO_HDL gpioHdl;


sem_t g_sendSem[NUM_OF_CH];
static int tx_size = 0;


void SleepUSec(long a_nUSec)
{
    struct timeval stTimeVal;

    stTimeVal.tv_sec = a_nUSec / 1000000;
    a_nUSec -= stTimeVal.tv_sec * 1000000;
    stTimeVal.tv_usec = a_nUSec;

    select(0, 0, 0, 0, &stTimeVal);
}


/* It sends the data with TXBUF_SIZE size through the sFPDP ch (="arg").
 */
void *send_thread(void* arg)
{
	int ch=0;
	int i; 
	int rtn=0;	// index of transfer block int rtn = 0;
	int totsend=0;		// total size of send
	int cnt = 0;
	unsigned char tbuf[TX_FRMSIZE+128]; 

	ch = *(int *)arg;

	tbuf[0] = 0xaa;
	tbuf[1] = 0xaa;
	tbuf[2] = 0x55;
	tbuf[3] = 0x55;

	for( i=4; i<TX_FRMSIZE; i++) {
		tbuf[i] = (unsigned char ) (i%256);
	}

	while(1) {
		sem_wait( &g_sendSem[ch] );

		for( cnt=0; cnt < 1; cnt++) {

//		      rtn = sfpdp_send( ch, (char *)&tbuf[cnt*4096], 4096 );

			rtn = sfpdp_send( ch, (char *)tbuf, tx_size );
			if( !(0<rtn) ) {
				printf( "%d-th block(16KB), there is no data of sfpdp_send.\n", cnt );
				return NULL;
			}
//			printf( "Send rtn = %d \n", rtn );
			totsend += rtn;
			usleep( 1 );
		}

		printf( "CH[%d] Amount of send data: %d\n", ch, totsend );

	} // while

	return NULL;
}



void *recv_thread(void* arg)
{
	int ch = 0;
	int nIndex = 0;
	int nRxSize;
	unsigned char pRxBuf[4096 + 128];
	int recv_tot[4];

	ch = *(int *)arg;
	recv_tot[ch]=0;

	while(1) {
		printf(" Rx Start.. \n");
		nRxSize = sfpdp_recv_single( ch, pRxBuf, 4096 );
		printf(" RxSize = %d\n", nRxSize);
		if( nRxSize < 0 ) {
			/* read error */
			printf("Recv error in Ch[%d]: %d\n", ch, recv_tot[ch] );
			continue;
		}
		else if( nRxSize == 0 ) {
			continue;
		}
		recv_tot[ch] += nRxSize;

		for( nIndex = 0 ; nIndex < nRxSize ; nIndex++ ) {
			if(pRxBuf[nIndex] != (unsigned char)(nIndex % 256)) {
				printf(" Err at Index:%2d, Tx:0x%02X, Rx:0x%02X\n",
					nIndex, (char)(nIndex % 256), pRxBuf[nIndex] );
			}
		}
	}
	return NULL; 
}



void *gpio_thread(void* arg)
{
	int i = 0;
	int pin = 6;
	GEF_GPIO_PIN_BITMAP pinBitmap;
	GEF_STATUS gpioStatus;
	struct timeval timeout;

	pinBitmap = (0x0001 << pin);
	timeout.tv_sec = 30;	// timeout is 3 seconds
	timeout.tv_usec = 0;


	while(1) {
		getchar();
		gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );

		printf( "GPIO(pin#%d) interrupted or timeout(%d sec)..\n", pin, timeout.tv_sec );
		sleep(1);
/*	
		scGpioSetHigh7();
		scGpioSetLow7();
		SleepUSec(400*1);
*/
		for( i=0; i < NUM_OF_CH; i++ )	sem_post( &g_sendSem[i] );
	}
	return NULL;
}



int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};
	int chno; 
	int semIndex = 0;

	pthread_t p_send0;
	pthread_t p_send1;
	pthread_t p_send2;
	pthread_t p_send3;
	pthread_t p_gpio;

	int iSendThreadID0;
	int iSendThreadID1;
	int iSendThreadID2;
	int iSendThreadID3;
	int iGpioThreadID;

	pthread_t p_recv0;
	int iRecvThreadID0;

	if( argc != 3 ) printf( "Usage: loop-sfpdp [ch#] [tx_size]\n\n" );
        if( argc > 3 ) return -1;

        if( argc == 1 ) {
                while(1) {
                        printf( "Select the channel and size to transfer data..\n" );
                        printf( "Range of the Channel number is from 0 to 8: " );
                        scanf( "%d", &chno );
                        if( chno<0 || chno>8) {
                                printf( "Range of the channel number is over: %d\n", chno );
                                continue;
                        }
                        else break;
                }

                while(1) {
                        printf( "Size of Tx data (5 ~ 16384(16K) Byte): " );
                        scanf( "%d", &tx_size );
                        if( (tx_size <0) || (tx_size >4096) ) {
                                printf( "Range of the data is over: %d\n", tx_size );
                                continue;
                        }
                        else break;
                }
        }

        else if( argc == 2 ) chno = atoi( argv[1] );
        else if( argc == 3 ) {
                chno = atoi( argv[1] );
                tx_size = atoi( argv[2] );
        }
        else    return -1;

        printf( "Execute: loop-sfpdp %d(ch#) %d(TX size) through SFPDP loopback..\n", chno, tx_size );


	scGpioInit();

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ ) {
		if( sem_init( &g_sendSem[semIndex], 0, 0) == -1) {
			printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
			return -1;
		}
	}

	if( sfpdp_init() != 0 ) return -1;
	printf("sfpdp_init \n");
/*
	if( sfpdp_start( 0, RXMODE_WITHOUT_PATTERN ) != 0 ) return -1;
*/
	if( sfpdp_start( 0, RXMODE_AFTER_PATTERN ) != 0 ) return -1;
	printf("sfpdp_start.\n");

	iGpioThreadID = pthread_create( &p_gpio, NULL, gpio_thread, NULL);
	iSendThreadID0 = pthread_create( &p_send0, NULL, send_thread, (void *)&chno );
	iRecvThreadID0 = pthread_create( &p_recv0, NULL, recv_thread, (void *)&chno );

//	iSendThreadID1 = pthread_create( &p_send1, NULL, send_thread, (void *)&chNum[1] );
//	iSendThreadID2 = pthread_create( &p_send2, NULL, send_thread, (void *)&chNum[2] );
//	iSendThreadID3 = pthread_create( &p_send3, NULL, send_thread, (void *)&chNum[3] );

	pthread_join(p_gpio, NULL);
	pthread_join(p_send0, NULL);
	pthread_join(p_recv0, NULL);
/*
//	pthread_join(p_send1, NULL);
//	pthread_join(p_send2, NULL);
//	pthread_join(p_send3, NULL);
*/
	scGpioClose();

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ )
	       	sem_destroy( &g_sendSem[semIndex] );

	sfpdp_close();

	return 0;
}

