/*
 * This is functions to read register values of each SFPDP Channel.
 * At 6 May 2021, it was modified by Jeongki Kim. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include "sfpdp-api.h"

static char *sfpdpreg_desc[] = {
	"Production Rev.",
	"Build Date.",
	"Serial Num.",
	"Master Control reg.\n\tB(0) 1: Reset, 0: Normal operation.",
	"Heart bit per sec. (Not Used)",		// 0x10
	"Not Used.",
	"Control Reg #1.\n\tB(3) Loopback Pall. B(2) Loopback Serial. B(1) Enable RX. B(0) Enabble TX.\n"
	"\tB(7) Copy Loop En.  B(6) Copy Master En.  B(5) Copy Mode En. B(4) CRC En.\n"
	"\t...\n"
	"\tB(23) RX to TX Flow Control. B(22) TX Clock En. B(21) RX Clock En. B(20) Clear Disparity Err Cnt.\n"
	"\tB(27) SWDV En. B(26) Status of Optic RX. B(25) S/W RX ready. B(24) After Match Pattern, RX Data.",
	"Control Reg #2.",
	"Link RX Packet drop cnt.",	// 0x20
	"Link RX Packet Pass cnt.",
	"Link RX CRC Err cnt.",
	"Link RX Disparity Err cnt.",
	"Link TX Packet pass cnt.",	// 0x30
	"TX DMA Packet cnt.",
	"TX DMA Byte cnt.",
	"RX DMA Packet cnt.",
	"RX DMA Byte cnt.",		// 0x40
	"Match Pattern 0x5555AAAA.",
	"DMA Receive Size, 0x800 (= 2048 B) default.",
	"Timeout between Packets, 0x47e (1150/200MHz-clk =5.75us min)."
};


static char *ctlreg_desc[] = {
	"Rev. Num.",
	"Build Date.",
	"Serial Num.",
	"IRQ Mask (Pending).",
	"IRQ En (B(0):0-> Ch0 TX En, B(4):0-> Ch0 RX En).",	// 0x10
	"IRQ Occur, Initially 0xFF is Clear.",			// 0x14
	"...",
	"..."
};


int main( int argc, char **argv )
{
	int fd, chno;		// channel number to view
	char devfilename[200];
	PCIDRV_INFO pi;
	unsigned long timeout;
	int i;

	if( argc != 2 ) printf( "!Usage: %s [ch#]\n", argv[0] );
	if( argc > 2 ) return -1; 

	while(1) {
		if( argc == 1 ) {
			printf( "\nSelect the SFPDP channel you want to view its register values. \n" );
			printf( "Range of the Channel number is from 0 to %d: ", (NUM_OF_DMA-1) );
			scanf( "%d", &chno );
		}
		else if( argc == 2 ) chno = atoi( argv[1] ); 
		else	return -1; 

		if( (chno<0) || (chno>(NUM_OF_DMA-1)) ) {
			printf( "Range of the channel number is over. \n" );
			return -1; 
		}
		else break;
	}

	printf( "Excute: %s Ch#%d..\n", argv[0], chno );

	sprintf( devfilename, "/dev/%s%d", SFPDP_DEV_NAME, chno );
	fd = open( devfilename, O_RDWR|O_SYNC );
	if( !(fd>0) ) {
		printf( "Devide file(%s) open fail.\n", devfilename );
		return -1; 
	}
	printf( "**device file(%s) open ok!\n\n", devfilename );
	printf( "\t Offset  Value      Description \n" );
        printf( "---------------------------------------------\n" );

        for( i=0; i<20; i++) {
                pi.addr = CH_FIRST_ADDR + (0x10000*chno) + (i*4);
                ioctl(fd, IOCTL_GET_REG, &pi);
		printf( "SFPDP[%d] 0x%02X 0x%08X %s\n", chno, pi.addr, pi.val, sfpdpreg_desc[i] );
        }
        printf( "---------------------------------------------\n\n" );

	printf( "\t Offset  Value      Description \n" );
        printf( "---------------------------------------------\n" );

        for( i=0; i<8; i++) {
                pi.addr = CTL_REG_ADDR + (i*4);
                ioctl(fd, IOCTL_GET_REG, &pi);
		printf( "IRQ Reg: 0x%2X 0x%08X %s\n", pi.addr, pi.val, ctlreg_desc[i] );
        }

	close(fd);
	printf("**close()\n");
	return 0;
}


