/*
 * This is functions to control GPIO signal. 
 * Initialize, Close, etc.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>

#include "gefcmn_gpio.h"

#define SBC627_GPIO_PIN_0 (0x0001)
#define SBC627_GPIO_PIN_1 (0x0002)
#define SBC627_GPIO_PIN_2 (0x0004)
#define SBC627_GPIO_PIN_3 (0x0008)
#define SBC627_GPIO_PIN_4 (0x0010)
#define SBC627_GPIO_PIN_5 (0x0020)
#define SBC627_GPIO_PIN_6 (0x0040)
#define SBC627_GPIO_PIN_7 (0x0080)

unsigned int nCallbackCountPin0 = 0;
unsigned int nCallbackCountPin1 = 0;
unsigned int nCallbackCountPin2 = 0;
unsigned int nCallbackCountPin3 = 0;


GEF_GPIO_HDL		gpioHdl		= NULL;
GEF_STATUS		gpioStatus	= GEF_STATUS_SUCCESS;
GEF_GPIO_MODE		gpioMode	= GEF_GPIO_MODE_INTERRUPT_EDGE;
GEF_GPIO_POLARITY	gpioPolarity	= GEF_GPIO_POLARITY_ACTIVE_HIGH;
GEF_GPIO_PIN_BITMAP	gpioPinBitmap	= 0x0;

unsigned int		pinCount	= 0;
// unsigned int 	dwIndex;

int scGpioInit();
int scGpioSet();
int scGpioClose();
void scGpioSetHigh7(void);
void scGpioSetLow7(void);


/* GPIO pin 7 is made to high and low */
void scGpioWrite(void)
{
	gpioPinBitmap |= SBC627_GPIO_PIN_7;
	
	gpioStatus = gefGpioWrite(gpioHdl, gpioPinBitmap);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioWrite() failed\n");
		gefGpioClose(gpioHdl);
		return;
	}

	gpioPinBitmap &= ~(SBC627_GPIO_PIN_7);
	gpioStatus = gefGpioWrite(gpioHdl, gpioPinBitmap);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioWrite() failed\n");
		gefGpioClose(gpioHdl);
		return ;
	}
}


void scGpioSetHigh7(void)
{
	gpioPinBitmap |= SBC627_GPIO_PIN_7;
	gpioStatus = gefGpioWrite(gpioHdl, gpioPinBitmap);
}


void scGpioSetLow7(void)
{
	gpioPinBitmap &= ~(SBC627_GPIO_PIN_7);
	gpioStatus = gefGpioWrite(gpioHdl, gpioPinBitmap);
}


void scGpioSetHigh6(void)
{
	gpioPinBitmap |= SBC627_GPIO_PIN_6;
	gpioStatus = gefGpioWrite(gpioHdl, gpioPinBitmap);
}


void scGpioSetLow6(void)
{
	gpioPinBitmap &= ~(SBC627_GPIO_PIN_6);
	gpioStatus = gefGpioWrite(gpioHdl, gpioPinBitmap);
}


int scGpioPinInterptInit( int pin )
{ 
	/* Open the GPIO library */ 
	gpioStatus = gefGpioOpen(&gpioHdl); 
	if (gpioStatus != GEF_STATUS_SUCCESS) {
		printf("ERROR: gefGpioOpen() failed\n"); 
		return 1; 
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, pin, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", pin );
		gefGpioClose(gpioHdl);
		return 1;
	}

	printf( "Gpio Pin:%d Interpt Init.. OK.\n", pin );
	return 0;
}



int scGpioInit()
{ 
	/* Open the GPIO library */ 
	gpioStatus = gefGpioOpen(&gpioHdl); 
	if (gpioStatus != GEF_STATUS_SUCCESS) {
		printf("ERROR: gefGpioOpen() failed\n"); 
		return 1; 
	}

	/* Get PinCount to Validate the GPIO pin number */
	gpioStatus = gefGpioGetPinCount(gpioHdl, &pinCount);
	if (gpioStatus != GEF_STATUS_SUCCESS) {
		printf("ERROR: gefGpioGetPinCount() failed\n"); 
		gefGpioClose(gpioHdl); 
		return 1;
	} 

	/* pin 6 is output */

	gpioMode= GEF_GPIO_MODE_OUTPUT;
	gpioPolarity= GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 6, gpioMode, gpioPolarity);

	if (gpioStatus != GEF_STATUS_SUCCESS) {
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 6);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode= GEF_GPIO_MODE_OUTPUT;
	gpioPolarity= GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 7, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 7);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 0, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 0);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 1, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 1);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 2, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 2);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 3, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 3);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 4, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 4);
		gefGpioClose(gpioHdl);
		return 1;
	}

	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_LOW;
//	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 5, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", 5);
		gefGpioClose(gpioHdl);
		return 1;
	}

/* pin 6 is input
	gpioMode = GEF_GPIO_MODE_INTERRUPT_EDGE;
	gpioPolarity = GEF_GPIO_POLARITY_ACTIVE_HIGH;
	gpioStatus = gefGpioSetConfiguration(gpioHdl, 6, gpioMode, gpioPolarity);
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
		printf("ERROR: gefGpioSetConfiguration(pin = %d) failed\n", dwIndex );
		gefGpioClose(gpioHdl);
		return 1;
	}
*/
   
	gpioStatus = gefGpioWrite(gpioHdl, 0);		// reset the status of pin to the value.
	if (gpioStatus != GEF_STATUS_SUCCESS)
	{
	   printf("ERROR: gefGpioWrite() failed\n");
	   gefGpioClose(gpioHdl);
	   return 1;
	}

	printf("Gpio Init... OK!\n");
	return 0;
}


int scGpioSet()
{
	return 0;
}


int scGpioClose()
{
	/* Close the GPIO library */
	gpioStatus = gefGpioClose(gpioHdl);

	if (gpioStatus != GEF_STATUS_SUCCESS) {
		printf("ERROR: gefGpioClose() failed\n");
		return -1;
	}
 
	printf("Gpio Close... OK!\n");
	return 0;
}

