/*
 * send data through sFPDP. 
 * test: 16M data send.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "sfpdp-api.h"


#define TXBUF_LSIZE	(4*1024*1024)		// 4M x 4 = 16MB
/*
#define TXBUF_SIZE	TENK_BLK
#define BUF_SIZE	((256*4))
#define BUF_SIZE	30720
#define TXNUM_BLK	8	// the number of Tx data
*/

#define WRITE_COUNT	1
#define INF_LOOP	1
#define NUM_OF_CH	4

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);

extern GEF_GPIO_HDL gpioHdl;

/*
unsigned int buff0[TXBUF_SIZE];	// 10240*4 sizeof(u_int)
unsigned int buff1[TXBUF_SIZE];
unsigned int buff2[TXBUF_SIZE];
unsigned int buff3[TXBUF_SIZE];
*/

sem_t g_sendSem[NUM_OF_CH];


void SleepUSec(long a_nUSec)
{
    struct timeval stTimeVal;

    stTimeVal.tv_sec = a_nUSec / 1000000;
    a_nUSec -= stTimeVal.tv_sec * 1000000;
    stTimeVal.tv_usec = a_nUSec;

    select(0, 0, 0, 0, &stTimeVal);
}


/* It sends the data with TXBUF_SIZE size through the sFPDP ch (="arg").
 */
void *send_thread(void* arg)
{
	int ch=0;
        int i; 
	int rtn=0;	// index of transfer block int rtn = 0;
	int totsend=0;		// total size of send
	int cnt = 0;
	int logstopflag = 0;
	unsigned long int checksum[4] = {0, 0, 0, 0};
	unsigned int *tbuf; 		// 4*1024*1024*sizeof(u_int)

	int fp;
	char logFile[80];
	unsigned int * ptr0, ptr1, ptr2, ptr3;

	ch = *(int *)arg;
/*
	initializes tbuf
 */
	tbuf = (unsigned int *) malloc( TXBUF_LSIZE * 4 ); 
	if( !tbuf ) {
		printf(" malloc error.. "); 
		return NULL; 
	}

	for( i=3; i<TXBUF_LSIZE; i++)
		tbuf[i] = i; 

	tbuf[0] = 0x5555aaaa;
	tbuf[1] = 0x5555aaaa;
	printf( "tbuf[0]=0x%08x, tbuf[i]=0x%08x\n", tbuf[0], tbuf[i-1] ); 
	
	while(1) {
		sem_wait( &g_sendSem[ch] );

		for( cnt=0; cnt < 1024; cnt++) {

			rtn = sfpdp_send( ch, (char *)&tbuf[cnt], 4096*4 );
			if( !(0<rtn) ) {
				printf( "%d-th block(16KB), there is no data of sfpdp_send.\n", cnt );
				free( tbuf ); 
				return NULL; 
			}
//			printf( "Send rtn = %d \n", rtn );
			totsend += rtn;
		}

		printf( "Amount of send data: %d\n", totsend );

	} // while
	free( tbuf ); 

	return NULL;
}



void *gpio_thread(void* arg)
{
	int i = 0;
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap;
	GEF_STATUS gpioStatus;
	struct timeval timeout;

	pinBitmap = (0x0001 << pin);
	timeout.tv_sec = 5;	// timeout is 3 seconds
	timeout.tv_usec = 0;

//	getchar();

	while(1) {
		gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );

		printf( "gpio: interrupted or timeout(%d sec)..\n", timeout.tv_sec );
		sleep(1);
	
//		scGpioSetHigh7();
//		scGpioSetLow7();
//		SleepUSec(400*1);
		for( i=0; i < NUM_OF_CH; i++ )	sem_post( &g_sendSem[i] );
	}
	return NULL;
}


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};
	int semIndex = 0;

	pthread_t p_send0;
	pthread_t p_send1;
	pthread_t p_send2;
	pthread_t p_send3;
	pthread_t p_gpio;

	int iSendThreadID0;
	int iSendThreadID1;
	int iSendThreadID2;
	int iSendThreadID3;
	int iGpioThreadID;

	system("rm -rf log*");

	scGpioInit();

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ ) {
		if( sem_init( &g_sendSem[semIndex], 0, 0) == -1) {
			printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
			return -1;
		}
	}

	if( sfpdp_init() != 0 ) return -1;
	printf("sfpdp_init \n");

	iGpioThreadID = pthread_create( &p_gpio, NULL, gpio_thread, NULL);
	iSendThreadID0 = pthread_create( &p_send0, NULL, send_thread, (void *)&chNum[0] );
//	iSendThreadID1 = pthread_create( &p_send1, NULL, send_thread, (void *)&chNum[1] );
//	iSendThreadID2 = pthread_create( &p_send2, NULL, send_thread, (void *)&chNum[2] );
//	iSendThreadID3 = pthread_create( &p_send3, NULL, send_thread, (void *)&chNum[3] );

	pthread_join(p_gpio, NULL);
	pthread_join(p_send0, NULL);
//	pthread_join(p_send1, NULL);
//	pthread_join(p_send2, NULL);
//	pthread_join(p_send3, NULL);

//	scGpioClose();

	for( semIndex=0; semIndex < NUM_OF_CH; semIndex++ )
	       	sem_destroy( &g_sendSem[semIndex] );

	sfpdp_close();

	return 0;
}

