/*
 * This is basic functions for sFPDP communication. 
 *
 * At 30 June 2021, modified by Jeongki Kim, SEEDCORE.
 *
 */
#include <stdio.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/ioctl.h>

#include "sfpdp-api.h"


int fd[4]={0,0,0,0};	/* device file id */

/* Not used 
struct data_buf {
	unsigned char	rdata[RX_FRMSIZE+16];
	int		len;
};
*/


char RBfrm[NUM_OF_DMA][RX_FRMSIZE+16];		// RX buff with a frame size
int  RBsize[NUM_OF_DMA] = {0,0,0,0};		// data size in a RX Buf
int  RBindx[NUM_OF_DMA] = {0,0,0,0};		// current data index in a RX Buf.


#ifdef _DEBUG_
void print_buff( unsigned char *pbuf, int sz )
{
	int i;

	if( sz > 999 ) printf( "Recommend to use print_bigbuff() function..\n");
	else printf("\n"); 

	for( i=0; i<sz; i++) {
		if( (i%16)== 0 ) printf( "Buf[%3d]", i );
		printf( " %02X", pbuf[i] );
		if( (i%16)== 15 ) printf( "\n" );
	}
	printf( "\n" );

}


void print_bigbuff( unsigned char *buf, int sz )
{
	int i=0;

	if( sz < 32 ) {
		printf( "Size is too small. Recommend to use print_buff() function..\n");
		return ;
	}
	printf("\n"); 

	for( i=0; i <16; i+=8 ) {
		printf("buf[%2d] %02X %02X %02X %02X %02X %02X %02X %02X  ", i,
			buf[i], buf[i+1], buf[i+2], buf[i+3], buf[i+4], buf[i+5], buf[i+6], buf[i+7] );
	}
	printf("...\n");

	for( i=(sz-16); i <sz; i+=8 ) {
		printf("buf[%2d] %02X %02X %02X %02X %02X %02X %02X %02X  ", i,
			buf[i], buf[i+1], buf[i+2], buf[i+3], buf[i+4], buf[i+5], buf[i+6], buf[i+7] );
	}
	printf("\n");
}
#endif



int sfpdp_init_control( int ch, unsigned int mode )
{
	PCIDRV_INFO pi;

#ifdef _DEBUG_
	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __func__ );
		return -1;
	}
#endif

	pi.addr = CH_FIRST_ADDR + (0x10000*ch) + 0x18;	/* Control Reg per Ch */
	pi.val = 0x03600003;				/* default value. */

	if( mode )	pi.val |= (mode);
#ifdef _DEBUG_
	printf("[%s]DBG> Offset 0x18 Val:0x%08X.\n", __func__, pi.val );
#endif

	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	sc_wmb();					// Write Memory Barrior.

	pi.addr = CH_FIRST_ADDR + (0x10000*ch) + 0x0C;	// Master Control Register
	pi.val = 0x01;					// Reset the sFPDP channel
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	sc_wmb();

	pi.val = 0x00;					// Normal operations
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	sc_wmb();

/* Offset 0x4C : Timeout Reg.
 * if (Delay between packets > Timeout), at the end of the packet, "DEAD_BEEF" is inserted. 
 * Default value is 0x47E (=1150 = 5.75us).
	pi.addr = CH_FIRST_ADDR + (0x10000*ch) + 0x4C;
	pi.val = 0x8FC;				// 0x8FC =2300d =11.5 usec.
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
*/

	return 0;
}



/* It clears BD buffer in which garbages was remainded */
int sfpdp_bdbuf_clear( int ch )
{
	int ret;
	int utimeout =20;		// micro sec.
	char buf[RX_FRMSIZE+16];

	if( !(0<fd[ch]) ) {
		printf( "[%s] ch[%d]: fd is not opened..", __func__, ch );
		return -1;
	}

	/* to extract some garbage data from DMA buffers */
	while(1) {

		ret = read( fd[ch], buf, utimeout );
		if( ret<0 ) {
			printf("[%s] ch[%d] Read_error.\n", __func__, ch );
			return -1;
		}
		if( ret>0 ) continue;   // garbages remained.
		break;
	}
	return 0;
}


int sfpdp_all_bdbuf_clear()
{
	int ich, rtn;

	for( ich=0; ich<NUM_OF_DMA; ich++) {
		rtn = sfpdp_bdbuf_clear( ich );
		if( rtn < 0) return -1;
	}
	return 0;
}



/* It sends data with "size".
 * return: the size of real sent data */
int sfpdp_send( int ch, char* buf, int size )
{
	int cframe=0;	// count of frame
	int tsize = size;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	/* Originally write() function return the size of the real written data,
	 * but this write() (=pci_write()) return 0 when it successes, return -1 when fail. */
	/* When size > TX_FRAMESIZE, the function that splits the data to a frame size.
	 */ 

	while( tsize > TX_FRMSIZE ) {
		if( write( fd[ch], buf, TX_FRMSIZE ) != 0 ) return -1;
		tsize -= TX_FRMSIZE;
		cframe++;
	}
	if( write( fd[ch], buf, tsize ) != 0 ) return -1;

#ifdef _DEBUG_
//	printf("[%s]DBG> TX Amount per write:%d.\n", __func__, size );
#endif

	return size;
}



/* When this function sends data, the last frame is sent with the SWDV signal. 
 * SWDV: Synch With Data Valid. */

int sfpdp_send_with_swdv( int ch, char* buf, int size )
{
	unsigned int pival;
	int rtn;
	int idx = 0;	// the number of write() function
	PCIDRV_INFO pi;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	while( size > 0 ) {
		if( size < TX_FRMSIZE ) {	/* this is the last frame */
			pi.addr = CH_FIRST_ADDR + (0x10000*ch) + 0x18;
			ioctl( fd[ch], IOCTL_GET_REG, &pi);
			pival = pi.val;
#ifdef _DEBUG_
		printf("[%s]DBG> Offset 0x18 Val:0x%08X.\n", __func__, pival );
#endif
			pi.val = (pival | SFPDP_TX_SWDV_MASK);
			ioctl( fd[ch], IOCTL_SET_REG, &pi);
		}

		/* this write() (=pci_write()) return 0 when it successes, return -1 when fail.
		 * Originally write() return the size of the real written data. */

		rtn = write( fd[ch], &buf[(TX_FRMSIZE)*idx], (size>TX_FRMSIZE ? TX_FRMSIZE : size) );
		if( rtn < 0) {
			printf( "[%s] write error in.\n", __FUNCTION__ ); 
			return -1;
		}
		size -= TX_FRMSIZE;
		idx++;
#ifdef _DEBUG_
		printf("[%s]DBG> TX Amount per write:%d.\n", __func__, (size + TX_FRMSIZE) );
#endif
	}

	return ( TX_FRMSIZE * idx + size);	// size is now negative.
}


/* It receives data with "size".
 * It waits until recving the "size" data if "usertimeout" is 0, 
 * otherwise, it returns only the data received when "usertimeout" is over.
 * return: the size of one block requested by an user.
 */
int sfpdp_recv( int ch, char* buf, int size, int usrtimeout)
{
	int reqsize = size;	// reqsize: requested size to return to user.
	int rtnindx =0;		// current index location of retrun block.
	int rbinx;
	char tbuf[RX_FRMSIZE+16];

	if( !(fd[ch]>0) | (size > RX_BUFSIZE) ) {
		printf( "[%s]>fd[%d] open error or size is over.\n", __func__, ch );
		return -1;
	}

	if( RBsize[ch] == 0 ) {
		RBsize[ch] = sfpdp_recv_single( ch, tbuf, usrtimeout );
		memcpy( (void*)&RBfrm[ch][0], (void*)tbuf, RBsize[ch] );
		RBindx[ch] = 0;
	}
#ifdef _DEBUG_
	printf("[%s]FrstRX Ch[%d]> rtnindx:%d, reqsize:%d, RBindx:%d, RBsize:%d.\n", __func__,
		       ch, rtnindx, reqsize, RBindx[ch], RBsize[ch] );
#endif

	while( reqsize > RBsize[ch] ) {
		rbinx = RBindx[ch];
		memcpy( (void*)&buf[rtnindx], (void*)&RBfrm[ch][rbinx], RBsize[ch] );
		rtnindx += RBsize[ch];
		reqsize -= RBsize[ch];

		RBsize[ch] = sfpdp_recv_single( ch, tbuf, usrtimeout );
		memcpy( (void*)&RBfrm[ch][0], (void*)tbuf, RBsize[ch] );
		RBindx[ch] = 0;
#ifdef _DEBUG_
//		printf("[%s]IntRX Ch[%d]> rtnindx:%d, reqsize:%d, RBindx:%d, RBsize:%d.\n", __func__,
//		       ch, rtnindx, reqsize, RBindx[ch], RBsize[ch] );
#endif
	}

	rbinx = RBindx[ch];
	memcpy( (void*)&buf[rtnindx], (void*)&RBfrm[ch][rbinx], reqsize );
	RBindx[ch] += reqsize;
	RBsize[ch] -= reqsize;

#ifdef _DEBUG_
	printf("[%s]LastRX Ch[%d]> rtnindx:%d, reqsize:%d, RBindx:%d, RBsize:%d.\n", __func__,
		       ch, rtnindx, reqsize, RBindx[ch], RBsize[ch] );
#endif

	return size;
}


/* It receives data just one time, which size is < or = RX_FRMSIZE.
 * return: the size of real data be received. */

int sfpdp_recv_single( int ch, char* buf, int usrtimeout )
{
	char tbuf[RX_FRMSIZE+64];
	int ret; 
	unsigned int checkval;

	ret = read( fd[ch], tbuf, usrtimeout );
	if( (ret<0) | (ret>RX_FRMSIZE) ) {
		printf("[%s] Read error.\n", __func__ );
		return -1;
	}

	if( ret > sizeof(int) ) {		// to remove 0xDEADBEEF.
		memcpy( (void*)&checkval, (void*)&tbuf[ret-sizeof(int)], sizeof(int) );
		if( checkval == 0xDEADBEEF ) {
			ret -= sizeof(int);
			memcpy( (void*)&checkval, (void*)&tbuf[ret-sizeof(int)], sizeof(int) );
		}
	}

#ifdef _DEBUG_
//	printf("[%s]DBG> Recv Amount per read: %d.\n", __func__, ret );
//	printf("[%s]DBG> RX Check-sum value: 0x%X.\n", __func__, checkval );
#endif

	memcpy( buf, tbuf, ret );
	return ret; 
}



int sfpdp_ch_open( int ch )
{
	char devfile[128];

	sprintf( devfile, "/dev/%s%d", SFPDP_DEV_NAME, ch );
	fd[ch] = open( devfile, O_RDWR|O_SYNC );
	if( !(0<fd[ch]) ) {
		printf( "[%s] Dev file open() fail: %s.\n", __func__, devfile );
		return -1;
	}
	return 0;
}


int sfpdp_ch_close( int ch )
{
	close( fd[ch] );
}


int sfpdp_all_open()
{
	int ich;
	char devfile[128];

	for( ich=0; ich<NUM_OF_DMA; ich++) {
		sprintf( devfile, "/dev/%s%d", SFPDP_DEV_NAME, ich );
		fd[ich] = open( devfile, O_RDWR|O_SYNC );
		if( !(0<fd[ich]) ) {
			printf( "[%s] Dev file open() fail: %s.\n", __func__, devfile );
			return -1;
		}
	}
	return 0;
}


int sfpdp_all_close()
{
	int ich =0;

	for( ich=0; ich<NUM_OF_DMA; ich++)	close(fd[ich]);
	return 0;
}



int sfpdp_bd_reset()
{
	if( !(0<fd[3]) ) {
		printf( "fd[%d] is not opened in [%s]..", 3, __FUNCTION__ );
		return -1;
	}

	ioctl( fd[3], IOCTL_RESET, NULL );
	return 0;
}


