/*
 * It is to make data file to be sent through an SFPDP Card.
 * Input arg is data size.
 * At 7 May 2020, by Jeongki Kim.
#include <pthread.h>
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <sys/stat.h>
 */

#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write, usleep()
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK


#define MK_BUFSIZE	1024		// size of data block.
#define MAX_DSIZE	(1024*1024)	// size of total data.
#define SDFILE		"./tx_source.log"


/* It makes a data file to send. mfd: file id, dsize: total data size to store. */
int make_txdata( int mfd, int dsize )
{
	int ch=0;
	int i, rtn=0, cnt = 0;
	unsigned int	match_pattern = 0x5555AAAA;
	unsigned int	rever_pattern = 0xAAAA5555;
	unsigned int	chksum=0;		// for packet data integrity 
	unsigned char	tbuf[MK_BUFSIZE+16];	// 1 KB
	int count = 0;

	
	while( dsize > 0 ) {
		memcpy( (void*)&tbuf[0], (void*)&match_pattern, sizeof(int) );
		memcpy( (void*)&tbuf[4], (void*)&rever_pattern, sizeof(int) );
		tbuf[8] =cnt;

		for(i=9; i < (MK_BUFSIZE-sizeof(int)); i++) {
//			tbuf[i] = rand() % 10;
			tbuf[i] = i % 256;
			chksum += tbuf[i];
		}

		memcpy( (void*)&tbuf[i], (void*)&chksum, sizeof(int) );

		rtn = write( mfd, tbuf, MK_BUFSIZE );
#ifdef DEBUG
		if( !(0<rtn) ) {
			printf( "%d-th block, there is no data.\n", cnt );
		}
		printf( "In %d-th block, written data size: %d Bytes.\n", cnt, rtn );
#endif

		cnt++;
		dsize -= MK_BUFSIZE;
	}

	return 0;
}



int get_main_args( int argc, char **argv )
{
	int TXsize; 

	if( argc != 2 ) printf( "Usage: %s [TXsize]\n\n", argv[0] );
	if( argc > 2 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the data size to create..\n" );
			printf( "Size of Tx data (9 ~ %d Byte): ", MAX_DSIZE );
			scanf( "%d", &TXsize );

		}
		else if( argc == 2 ) {
			TXsize = atoi( argv[1] ); 
		}

		if( (TXsize <9) || (TXsize > MAX_DSIZE) ) {
			printf( "Range of the data is over: %d\n", TXsize );
			argc =1;
			continue; 
		}
		else break;
	}
	return TXsize;
}



int main(int argc, char **argv)
{
	int sdfd; 
	int sdsize; 
	char sdfilename[128];

	sdsize = get_main_args( argc, argv );

	printf( "Execute: %s %d(Data size) to a file..\n", argv[0], sdsize );

	sprintf( sdfilename, "%s", SDFILE );
	sdfd = open( sdfilename, O_WRONLY|O_CREAT );
	if( !(sdfd>0) ) {
		printf( "[%s] Data file open() fail: %s.\n", __func__, SDFILE );
		return -1;
	}

	make_txdata( sdfd, sdsize );

	close( sdfd );

	return 0;
}


