/*
 * This is functions to read register values. 
 */
#include <stdio.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <sys/types.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include "my_ioctl.h"

#if 0
#define XAXIDMA_TX_CR_OFFSET    0x00000000 /* Channel control */
#define XAXIDMA_TX_SR_OFFSET    0x00000004 /* Status */
#define XAXIDMA_DELAY_MASK      0xFF000000 /* Delay timeout counter */
#define XAXIDMA_COALESCE_MASK	0x00FF0000 /* Coalesce counter */

#define XAXIDMA_CR_RESET_MASK   0x00000004 /* Reset DMA engine */

#define XAXIDMA_DELAY_SHIFT	24
#define XAXIDMA_COALESCE_SHIFT	16

#define XAXIDMA_DFT_TX_THRESHOLD    32
#define XAXIDMA_DFT_TX_WAITBOUND    254
#define XAXIDMA_DFT_RX_THRESHOLD    32
#define XAXIDMA_DFT_RX_WAITBOUND    254

#define XAXIDMA_IRQ_ALL_MASK	0x00007000 /* All interrupts */
#endif

int main( int argc, char **argv )
{
	int fd, chno;		// channel number to view
	char devfilename[200];
	PCIDRV_INFO pi;
	unsigned long timeout;
	int i;

	if( argc != 2 ) printf( "!Usage: read-reg [ch#]\n\n" );

	if( argc > 2 ) return -1; 

	if( argc == 1 )
		do{
			printf( "Select the sFPDP channel you want to view its register values. \n" );
			printf( "Range of the Channel number is from 0 to 3: " );
			scanf( "%d", &chno );
			if( chno<0 || chno>3) printf( "Range of the channel number is over. \n" );
		} while( chno<0 || chno>3);

	else if( argc == 2 ) chno = atoi( argv[1] ); 
	else	return -1; 

	sprintf( devfilename, "/dev/scsfpdp%d", chno );
	fd = open( devfilename, O_RDWR|O_SYNC );
	if( !(fd>0) ) {
		printf( "Devide file(%s) open fail.\n", devfilename );
		return -1; 
	}
	printf( "**device file(%s) open ok!\n\n", devfilename );

	pi.addr = 0x50000 + (0x10000*chno) + 0x00;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Product Rev.\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x04;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Build Date\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x08;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Scrach PAD, R/W reg. \n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x0c;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Master Control reg.\n", chno, pi.addr, pi.val);
	printf( "\tBit(0)? 1: Reset, 0: Normal operation.\n" );

	pi.addr = 0x50000 + (0x10000*chno) + 0x10;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x ... \n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x14;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x ... \n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x18;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Control reg.#1\n", chno, pi.addr, pi.val );
	printf( "\tB(0): enableTX, B(1): enableRX,     B(2): Loopback serial, B(3): Loopback Pall,\n" ); 
	printf( "\tB(4): CRC en,   B(5): Copy Mode en, B(6): Copy Mast.en,    B(7): Copy Loop en,\n" ); 
	printf( "\tB(20): Clear Disparity err, B(21): RX clock en, B(22): TX clock en, B(23): Flow,\n"); 
	printf( "\tB(24): after 0x5555AAAA pattern match, RX receive. B(25): S/W RX ready(=1),\n"); 

	pi.addr = 0x50000 + (0x10000*chno) + 0x1C;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Control reg.#2\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x20;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x RX Packet Drop cnt.\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x24;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x(=%dd) RX Packet Pass cnt.\n", chno, pi.addr, pi.val, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x28;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x RX CRC err cnt.\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x2C;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x RX Disparity err cnt.\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x30;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x(=%dd) TX Packet pass cnt.\n", chno, pi.addr, pi.val, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x34;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x(=%dd) TX DMA Packet cnt.\n", chno, pi.addr, pi.val, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x38;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x(=%dd) TX DMA Byte cnt.\n", chno, pi.addr, pi.val, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x3C;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x(=%dd) RX DMA Packet cnt.\n", chno, pi.addr, pi.val, pi.val );

	pi.addr = 0x50000 + (0x10000*chno) + 0x40;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x(=%dd) RX DMA Byte cnt.\n", chno, pi.addr, pi.val, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x44;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Match Pattern 0x5555AAAA\n", chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x48;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x DMA Receive Size, 0x800 (= 2048 B) default\n",
		 chno, pi.addr, pi.val);

	pi.addr = 0x50000 + (0x10000*chno) + 0x4c;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "SFPDP[%d] 0x%02X 0x%08x Timeout between packets, 0x47e (1150/200MHz-clk =5.75us min)\n",
		 chno, pi.addr, pi.val);


	pi.addr = 0x90000 + 0x00;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ Base.\n", pi.addr, pi.val);

	pi.addr = 0x90000 + 0x04;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ ...\n", pi.addr, pi.val);

	pi.addr = 0x90000 + 0x08;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ ...\n", pi.addr, pi.val);

	pi.addr = 0x90000 + 0x0C;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ ...\n", pi.addr, pi.val);

	pi.addr = 0x90000 + 0x10;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ Mask (Pending)\n", pi.addr, pi.val);

	pi.addr = 0x90000 + 0x14;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ Status (Bits: ch enable)\n", pi.addr, pi.val);

	pi.addr = 0x90000 + 0x18;
	ioctl(fd, IOCTL_GET_REG, &pi);
	printf( "IRQ Reg: 0x%x 0x%08x IRQ ...\n", pi.addr, pi.val);

	close(fd);
	printf("**close()\n");
	return 0;
}

