#include <stdio.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <pthread.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include "seedcore_api.h"

#include "gefcmn_gpio.h"

#define BUFF_SIZE (1024*40)
#define SINGLE_DURATION_PKT_SIZE (40960*38)
#define GPIO_PKT_SIZE SINGLE_DURATION_PKT_SIZE

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow6(void);
extern int scGpioSetLow7(void);

extern GEF_GPIO_HDL gpioHdl;

static unsigned int frame_size[4] = {0,0,0,0};
static unsigned long int g_frame_size[4] = {0,0,0,0};
static int tx_stop_flag = 0;
static int rx_stop_flag = 0;
static int tick_stop_flag = 0;

unsigned int GetMSec()
{
	unsigned int uiCurTime;
	struct timeval tv;

	gettimeofday(&tv, NULL);

	uiCurTime = tv.tv_sec * 1000 + tv.tv_usec/1000;

	return uiCurTime;
}

void *timer_thread(void *arg)
{
	unsigned int uiStartTime = 0;
	unsigned int uiCurrTime = 0;
	int iTickInterVal = 1000;
	int uiTickCount = 0;
	int ch = 0;

	uiStartTime = GetMSec();

	printf("speed check start \n");
	while(1)
	{
//		if(tick_stop_flag)
//			break;
		uiCurrTime = GetMSec();
		if( (uiCurrTime - uiStartTime) >= iTickInterVal )
		{
			uiTickCount++;

			printf("[%d] Rx[0] %5.3f Gbps (%ld), Rx[1] %5.3f Gbps (%ld), Rx[2] %5.3f Gbps (%ld), Rx[3] %5.3f Gbps (%ld)\n", uiTickCount, g_frame_size[0] * 8   / 1000000000.f, frame_size[0], g_frame_size[1] * 8   / 1000000000.f, frame_size[1], g_frame_size[2] * 8   / 1000000000.f, frame_size[2], g_frame_size[3] * 8   / 1000000000.f, frame_size[3]);
			g_frame_size[0] = 0;
			g_frame_size[1] = 0;
			g_frame_size[2] = 0;
			g_frame_size[3] = 0;
			uiStartTime = uiCurrTime;
		}
		usleep(100000);
	}
	printf("speed check stop \n");
	return NULL;
}


void *send_thread(void *arg)
{
	unsigned int start_buff[2];
	char buff[BUFF_SIZE];
        unsigned int i = 0;
        unsigned int pattern = 0;
	int ch =  *(int *)arg;

	printf("ch[%d] Tx start \n", ch);
	for(i=0; i < BUFF_SIZE ; i++)
	{
		buff[i] = pattern;
#if 0
		printf("%d[0x%02x][%d] ", i, buff[i], buff[i]);
		if(!(i % 10))
			printf("\n");
#endif
		if(pattern / 127)
			pattern = 0;
		else
			pattern++;
        }
	buff[0] = 0x55;
	buff[1] = 0x55;
	
	while(1)
	{
//		if(tx_stop_flag) break;
		sfpdp_send(ch, buff, 8);
//		sfpdp_send(ch, buff, 1024);
//		usleep(1000);
		sleep(1);
//		break;
	}
	printf("ch[%d] Tx stop \n", ch);
	return;
}


void *recv_thread(void* arg)
{
	char buff[BUFF_SIZE];
	int ret;
	int ch =  *(int *)arg;
	int i;

	printf("ch[%d] Rx start \n", ch);
	memset(buff, 0, BUFF_SIZE);
	while(1)
	{
//		if(rx_stop_flag)	break;
		ret = sfpdp_recv(ch, buff, 40960);
#if 1
		if(buff[0] != 0x55) {
			for(i = 0; i < 50; i++) {
				if((i%5) == 0)
					printf("\n");
				printf("ch[%d] buff[%d] 0x%02x ", ch, i, buff[i]);
			}
			printf("\n\n");
		}
#endif
	//	printf("ch[%d] buff[0] 0x%02x  buff[1] 0x%02x  buff[2] 0x%02x  buff[3] 0x%02x buff[4] 0x%02x \n", ch, buff[0], buff[1], buff[2], buff[3], buff[4]);
		frame_size[ch] = 40960;
		g_frame_size[ch] += frame_size[ch];
		memset(buff, 0, BUFF_SIZE);
	}
	printf("ch[%d] Rx stop \n", ch);

	return NULL;
}



int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};

	char buff[40960];
	unsigned int i = 0;
	unsigned int pattern = 0;
	int ret;
	int fp;
	char logFile[80];

	pthread_t p_send0;
	pthread_t p_send1;
	pthread_t p_send2;
	pthread_t p_send3;
	pthread_t p_tick_thread;
	pthread_t p_recv0;
	pthread_t p_recv1;
	pthread_t p_recv2;
	pthread_t p_recv3;
	int iSendThreadID0;
	int iSendThreadID1;
	int iSendThreadID2;
	int iSendThreadID3;
	int iTickThreadID;
	int iRecvThreadID0;
	int iRecvThreadID1;
	int iRecvThreadID2;
	int iRecvThreadID3;

	system("rm -f log*");


	sfpdp_init();
	printf("sfpdp_init \n");

	sfpdp_start(0, 1);
	sfpdp_start(1, 1);
	sfpdp_start(2, 1);
	sfpdp_start(3, 1);
	printf("sfpdp_start \n");

//	iTickThreadID = pthread_create(&p_tick_thread, NULL, timer_thread, NULL);

//	iRecvThreadID0 = pthread_create(&p_recv0, NULL, recv_thread, (void *)&chNum[0]);
//	iRecvThreadID1 = pthread_create(&p_recv1, NULL, recv_thread, (void *)&chNum[1]);
//	iRecvThreadID2 = pthread_create(&p_recv2, NULL, recv_thread, (void *)&chNum[2]);
//	iRecvThreadID3 = pthread_create(&p_recv3, NULL, recv_thread, (void *)&chNum[3]);

//	getchar();
	iSendThreadID0 = pthread_create(&p_send0, NULL, send_thread, (void *)&chNum[0]);
//	iSendThreadID1 = pthread_create(&p_send1, NULL, send_thread, (void *)&chNum[1]);
//	iSendThreadID2 = pthread_create(&p_send2, NULL, send_thread, (void *)&chNum[2]);
//	iSendThreadID3 = pthread_create(&p_send3, NULL, send_thread, (void *)&chNum[3]);


#if 0
	getchar();
	tx_stop_flag = 1;

	getchar();
	rx_stop_flag = 1;

	getchar();
	tick_stop_flag = 1;
#endif

	pthread_join(p_send0, NULL);
//	pthread_join(p_send1, NULL);
//	pthread_join(p_send2, NULL);
//	pthread_join(p_send3, NULL);

//	pthread_join(p_recv0, NULL);
//	pthread_join(p_recv1, NULL);
//	pthread_join(p_recv2, NULL);
//	pthread_join(p_recv3, NULL);

//	pthread_join(p_tick_thread, NULL);

	printf("close \n");

	sfpdp_close();

	return 0;

}


